/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Auth Service
 */

import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { ConfigService } from '@shared/config/config.service';
import {
  IAuthLoginData,
  IAuthLoginResponseData,
  IAuthUserData,
} from './models';
import { of } from 'rxjs';
import { switchMap, tap } from 'rxjs/operators';
import jwt_decode from 'jwt-decode';

@Injectable()
export class AuthService {
  private _apiUrl: string;
  private _storageKey: string;

  constructor(private _config: ConfigService, private _http: HttpClient) {
    this._apiUrl = this._config.get('apiUrl') + '/auth';
    this._storageKey = this._config.get('authStorageKey');
  }

  // Receives a response with access_token, stores it, and then
  // returns observable with auth user data.
  logIn(loginData: IAuthLoginData): Observable<IAuthUserData> {
    const { email, password } = loginData;

    return this._http
      .post<IAuthLoginResponseData>(`${this._apiUrl}/login`, {
        email,
        password,
      })
      .pipe(
        switchMap((res) => {
          this.storeToken(res.token);
          return of(this.decodeToken(res.token));
        })
      );
  }

  logOut(id: string): Observable<any> {
    return this._http
      .post(`${this._apiUrl}/logout`, { id })
      .pipe(tap(() => localStorage.removeItem(this._storageKey)));
  }

  storeToken(token: string) {
    localStorage.setItem(this._storageKey, token);
  }

  loadUserData(): IAuthUserData | null {
    const token = localStorage.getItem(this._storageKey);
    return token ? this.decodeToken(token) : null;
  }

  decodeToken(token: string): IAuthUserData {
    const { id, email, profile, role, companyId, locationId }: any =
      jwt_decode(token);
    return { id, email, token, profile, role, companyId, locationId };
  }
}
