/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * AuthToken Interceptor.
 *
 * Interceptor used to inject the current user's token into all
 * HTTP requests as a Bearer token in the Authorization header.
 * It takes the token from the module's state if a user is logged in.
 */

import { Injectable } from '@angular/core';
import {
  HttpInterceptor,
  HttpEvent,
  HttpHandler,
  HttpRequest,
} from '@angular/common/http';
import { Observable } from 'rxjs';
import { first, mergeMap } from 'rxjs/operators';
import { Store, select } from '@ngrx/store';
import { selectors } from '../state';
import { IAuthUserData } from '../models';

@Injectable()
export class AuthTokenInterceptor implements HttpInterceptor {
  constructor(private _store: Store) {}

  intercept(
    req: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    // The token for the current user is stored in the Auth store.
    // If no user is logged in, the interceptor returns an observable
    // of the HTTP requests. If there is a user logged in, it creates
    // a new requests with the Authorization header and retuns an Observable
    // of it.
    return this._store.pipe(select(selectors.selectUser)).pipe(
      first(), // We want to take only the first emission.
      mergeMap((userData?: IAuthUserData) => {
        const newReq = userData?.token
          ? req.clone({
              headers: req.headers.set(
                'Authorization',
                `Bearer ${userData.token}`
              ),
            })
          : req;
        return next.handle(newReq);
      })
    );
  }
}
