import { EntityState, EntityAdapter, createEntityAdapter } from '@ngrx/entity';
import { createReducer, Action, on } from '@ngrx/store';
import * as actions from './actions';
import { CompanySelectItemDto, CompanyDto, CompanyListItemDto } from '../dto';
import { selectLoadedItem } from './selectors';

export const FEATURE_STATE_NAME = 'companies';

export interface State extends EntityState<CompanyListItemDto> {
  selectedItemId: string;
  sortOrder: string;
  loadingItems: boolean;

  loadingItem: boolean;
  loadedItem: CompanyDto;

  savingItem: boolean;
  deletingItem: boolean;

  total: number;
  searchingForSelection: boolean;
  foundForSelection: CompanySelectItemDto[];
}

export const adapter: EntityAdapter<CompanyListItemDto> = createEntityAdapter<CompanyListItemDto>(
  {
    sortComparer: false,
  }
);

const initialState: State = adapter.getInitialState({
  selectedItemId: null,
  sortOrder: 'ASC',
  loadingItems: false,
  loadingItem: false,
  loadedItem: null,
  savingItem: false,
  deletingItem: false,

  total: 0,
  searchingForSelection: false,
  foundForSelection: [],
});

const _entityReducer = createReducer(
  initialState,

  on(actions.selectItem, (state, { id }) => ({ ...state, selectedItemId: id })),

  on(actions.loadItems, (state) => ({
    ...state,
    loadingItems: true,
  })),
  on(actions.loadItemsSuccess, (state, { items }) =>
    adapter.setAll(items, { ...state, loadingItems: false })
  ),

  on(actions.loadItem, (state) => ({
    ...state,
    loadingItem: true,
  })),
  on(actions.loadItemSuccess, (state, { item }) => ({
    ...state,
    loadingItem: false,
    loadedItem: item,
  })),

  on(actions.loadTotalSuccess, (state, { total }) => ({ ...state, total })),

  on(actions.addItem, (state) => ({ ...state, savingItem: true })),
  on(actions.addItemSuccess, (state, { item }) =>
    adapter.addOne(item, {
      ...state,
      savingItem: false,
      selectedItemId: item.id,
    })
  ),

  on(actions.updateItem, (state) => ({ ...state, savingItem: true })),
  on(actions.updateItemSuccess, (state, { item }) =>
    adapter.updateOne(item, {
      ...state,
      savingItem: false,
      selectedItemId: item.id.toString(),
    })
  ),

  on(actions.deleteItem, (state) => ({ ...state, deletingItem: true })),
  on(actions.deleteItemSuccess, (state, { id }) =>
    adapter.removeOne(id, { ...state, deletingItem: false, loadedItem: null })
  ),

  on(actions.sortItems, (state, { sortOrder }) => ({
    ...state,
    sortOrder,
  })),

  on(actions.searchForSelection, (state) => ({
    ...state,
    searchingForSelection: true,
  })),
  on(actions.searchForSelectionSuccess, (state, { companies }) => ({
    ...state,
    searchingForSelection: false,
    foundForSelection: companies,
  }))
);

export function reducer(state: State | undefined, action: Action) {
  return _entityReducer(state, action);
}
