import {
  Component,
  Input,
  OnInit,
  OnChanges,
  SimpleChanges,
  ChangeDetectionStrategy,
} from '@angular/core';
import { Observable } from 'rxjs';
import { Store, select } from '@ngrx/store';
import { tap } from 'rxjs/operators';
import { actions, selectors } from '../../state';
import { actions as modalFormActions } from '@shared/modal-form';
import { actions as modalActions } from '@shared/modal';
import { LocationListItemDto } from '../../dto';
import { IMarker } from '@shared/ui/components/ui-map';
import { getColor } from '../colors';

@Component({
  selector: 'locations-table',
  templateUrl: 'locations-table.component.html',
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class LocationsTableComponent implements OnInit, OnChanges {
  @Input() companyId: string;

  items$: Observable<LocationListItemDto[]>;
  isLoading$: Observable<boolean>;
  selectedItem$: Observable<LocationListItemDto>;

  markers: IMarker[] = [];
  center: IMarker;

  readonly MODAL_CREATE_ID = 'locations_create_form';
  readonly MODAL_DELETE_CONFIRM_ID = 'locations_delete_confirm';

  constructor(private _store: Store) {}

  ngOnInit() {
    this.items$ = this._store.pipe(select(selectors.selectItems)).pipe(
      // Parse the items' content to create the markers for the map.
      tap((items) =>
        items.forEach((item) =>
          this.markers.push({
            id: item.id,
            name: item.name,
            shortText: '',
            url: `/locations/${item.id}`,
            coords: {
              lat: parseFloat(item.lat),
              lng: parseFloat(item.lng),
            },
          })
        )
      )
    );
    this.isLoading$ = this._store.pipe(select(selectors.selectIsLoadingItems));
    this.selectedItem$ = this._store
      .pipe(select(selectors.selectCurrentItem))
      .pipe(
        tap((item) => {
          if (item) {
            this.center = this.markers.find((marker) => marker.id === item.id);
          }
        })
      );
  }

  ngOnChanges(changes: SimpleChanges) {
    if (changes.hasOwnProperty('companyId')) {
      this.markers = [];
      this._store.dispatch(
        actions.loadItemsForCompany({ companyId: this.companyId })
      );
    }
  }

  showCreateForm() {
    this._store.dispatch(
      modalFormActions.openModalForm({ id: this.MODAL_CREATE_ID })
    );
  }

  selectLoaction(id: string) {
    this._store.dispatch(actions.selectItem({ id }));
  }

  showConfirmDelete() {
    this._store.dispatch(
      modalActions.openModal({ id: this.MODAL_DELETE_CONFIRM_ID })
    );
  }

  getTypeColor(item: LocationListItemDto): string {
    return getColor(item.type);
  }
}
