/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * CRUDIndexPage abstract class.
 *
 * This class provides common functionality to components that
 * implement an index page. It provides basic methods to control
 * listing, sorting, searching, creation, update and deletion
 * of items. When extending from it, a service of type ICRUDService
 * must be injected in the constructor of the derived class. This
 * service will be used to handle the CRUD operations.
 *
 * Also, the derived class must inject a service of the type
 * IModalService, which will be used to show dialogs for creating,
 * editing and deleting operations.
 *
 * This class provides two observables that can be accessed by derived
 * classes:
 *
 * 1. items$: contains the items loaded in the list.
 * 2. selectedItem$: contains the item currently selected in the list.
 */

import { Observable } from 'rxjs';
import {
  CRUDService,
  CRUDServiceOrder,
  CRUDServiceStatusType,
} from './CRUDService';
import { IModalService } from '@shared/modal/modal.interface';

export enum CRUDModalIdType {
  ADD_NEW = 'modal_add_new',
  EDIT = 'modal_edit',
  CONFIRM_DELETE = 'modal_confirm_delete',
}

export abstract class CRUDIndexPage<T> {
  // The CRUDService and ModalService must be injected
  // by the derived class.
  protected _crudService: CRUDService<T>;
  protected _modalService: IModalService;

  // Accessors to enums for the templates.
  eStatusType = CRUDServiceStatusType;
  eCRUDModalId = CRUDModalIdType;

  // Observable to the status of the service.
  // This can be used to show loaders when CRUD operations
  // are in exectuion.
  status$: Observable<CRUDServiceStatusType>;

  // Observables to the the current loaded items and
  // selected item.
  items$: Observable<T[]>;
  selectedItem$: Observable<T>;

  /**
   * Links to observables from the CRUD service.
   */
  init() {
    this.status$ = this._crudService.status$;
    this.items$ = this._crudService.items$;
    this.selectedItem$ = this._crudService.selectedItem$;
  }

  /**
   * Shows a modal dialog.
   *
   * @param id Value of the ID property passed to the modal.
   */
  showModal(id: string) {
    this._modalService.open(id);
  }

  /**
   * Helper to show a modal dialog with the create form.
   * The modal tag in the template should have its id attribute
   * set to the corresponding id type.
   */
  showAddNewForm() {
    this._modalService.open(CRUDModalIdType.ADD_NEW);
  }

  /**
   * Helper to show a modal dialog with the edit form.
   * The modal tag in the template should have its id attribute
   * set to the corresponding id type.
   */
  showEditForm() {
    this._modalService.open(CRUDModalIdType.EDIT);
  }

  /**
   * Helper to show a modal dialog asking to confirm deletion of item.
   * The modal tag in the template should have its id attribute
   * set to the corresponding id type.
   */
  showConfirmDelete() {
    this._modalService.open(CRUDModalIdType.CONFIRM_DELETE);
  }

  /**
   * Sets the given item as the selected one in the CRUD service.
   *
   * @param id ID of the item to select.
   */
  handleSelectedItem(id: string) {
    this._crudService.selectItem(id);
  }

  /**
   * Handles changes in the search box of the list that contains
   * the items.
   *
   * @param text Text in the search box.
   */
  handleSearchChange(text: string) {
    // TODO: implement handleSearchChange
    alert(`TODO: Handle search for text ${text}`);
  }

  /**
   * Handles changes in the order direction of the items.
   *
   * @param order Order direction.
   */
  handleOrderItems(order: CRUDServiceOrder) {
    this._crudService.orderItems(order);
  }

  /**
   * Handles the submission of values by the form used to create
   * new items. The form will only submit if its validation is successful.
   *
   * @param values Values submitted by the form.
   */
  handleAddNewSubmit({ formValues, resetForm, closeModal }) {
    this._crudService.create(formValues).subscribe(() => {
      closeModal();
      resetForm();
    });
  }

  /**
   * Handles the submission of values by the form used to update
   * new items. The form will only submit if its validation is successful.
   *
   * @param values Values submitted by the form.
   */
  handleEditSubmit(id: string, values: T) {
    this._crudService
      .update(id, values)
      .subscribe(() => this._modalService.close());
  }

  /**
   * Handles the deletion of a given item.\
   *
   * @param cb Callback to close the dialog.
   * @param id ID of the item to delete.
   */
  handleDeleteItem(cb: () => void, id: string) {
    this._crudService.delete(id).subscribe(() => cb());
  }

  /**
   * Checks if the list of items is loading items.
   *
   * @param status Current status of the service.
   * @returns `true` if the list is loading items.
   */
  isListBusy(status: CRUDServiceStatusType): boolean {
    return status === CRUDServiceStatusType.ITEMS_LOADING;
  }

  /**
   * Checks if the view of the selected item is loading the item.
   * @param status Current status of the service.
   * @returns `true` if the view is loading the item.
   */
  isViewBusy(status: CRUDServiceStatusType): boolean {
    return status === CRUDServiceStatusType.ITEM_LOADING;
  }

  /**
   * Checks if the the selected item is being deleted.
   * @param status Current status of the service.
   * @returns `true` if the item is being deleted.
   */
  isDeleting(status: CRUDServiceStatusType): boolean {
    return status === CRUDServiceStatusType.ITEM_DELETING;
  }

  /**
   * Checks if an item is being saved (added or updated).
   * @param status Current status of the service.
   * @returns `true` if the item is being saved.
   */
  isSaving(status: CRUDServiceStatusType): boolean {
    return (
      status === CRUDServiceStatusType.ITEM_CREATING ||
      status === CRUDServiceStatusType.ITEM_UPDATING
    );
  }
}
