import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';

export class CRUDService<DtoType, WriteDtoType> {
  protected http: HttpClient;
  protected endpoint: string;

  constructor(http: HttpClient, endpoint: string) {
    this.http = http;
    this.endpoint = endpoint;
  }

  getAll(search?: string, order?: string, countOnly = false): Observable<any> {
    const params = {
      search: search || '',
      order: order || 'ASC',
      countOnly: countOnly.toString(),
    };
    return this.http.get<DtoType>(`${this.endpoint}/withTotals`, { params });
  }

  getFor(
    fieldName: string,
    fieldValue: string,
    fields?: string
  ): Observable<any> {
    const params = {
      filter: `${fieldName}||$eq||${fieldValue}`,
      fields: fields || 'id,name',
    };
    return this.http.get(`${this.endpoint}`, {
      params,
    });
  }

  getAllSimple(search?: string, fields?: string): Observable<any> {
    const params = {
      filter: 'name||$cont||' + search,
      fields: fields || 'id,name',
    };
    return this.http.get(`${this.endpoint}`, {
      params,
    });
  }

  getCount(): Observable<number> {
    return this.http.get<number>(`${this.endpoint}/count`);
  }

  /**
   * Fetches one item from the backend.
   *
   * @param id - ID of the item to fecth.
   * @returns Observable to the item.
   */
  getOne(id: string, params?: any): Observable<any> {
    return this.http.get<DtoType>(`${this.endpoint}/${id}`, { params });
  }

  /**
   * Submits a new item to the backend with a POST call.
   * At the moment, this will refetch items from the backend after
   * the post operation is finished.
   *
   * @param newItem Item data to submit.
   * @returns Observable.
   */
  create(newItem: WriteDtoType): Observable<any> {
    return this.http.post(this.endpoint, newItem);
  }

  /**
   * Updates an item.
   * At the moment, this will refetch items from the backend after
   * the post operation is finished.
   *
   * @param id ID of the item to update.
   * @param data Data to update the item with.
   * @returns Observable
   */
  update(id: string, data: WriteDtoType): Observable<any> {
    return this.http.patch(`${this.endpoint}/${id}`, data);
  }

  /**
   * Delete an item in the backend given its id.
   * At the moment, this will refetch items from the backend after
   * the post operation is finished.
   *
   * @param id ID of the item to delete.
   * @returns Observable.
   */
  delete(id: string): Observable<any> {
    return this.http.delete(`${this.endpoint}/${id}`);
  }
}
