import {
  Component,
  Input,
  Output,
  OnInit,
  OnDestroy,
  EventEmitter,
} from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Observable, Subscription } from 'rxjs';
import { Store, select } from '@ngrx/store';
import { selectors } from '../state';
import { actions } from '../state';

import {
  transition,
  trigger,
  state,
  style,
  animate,
  animateChild,
  query,
} from '@angular/animations';

@Component({
  selector: 'modal-form',
  templateUrl: 'modal-form.component.html',
  styleUrls: ['modal-form.component.scss'],
  animations: [
    trigger('ngIfAnimation', [
      transition(':enter, :leave', [query('@*', animateChild())]),
    ]),
    trigger('slideInOut', [
      state('*', style({ transform: 'translateX(-100%)' })),
      state('void', style({ transform: 'translateX(100%)' })),
      transition(':enter', animate('350ms ease')),
      transition(':leave', animate('350ms ease')),
    ]),
  ],
})
export class ModalComponent implements OnInit, OnDestroy {
  @Input() form: FormGroup;
  @Input() id: string;
  @Input() title = '';
  @Input() submitLabel = 'Save';
  @Input() cancelLabel = 'Cancel';
  @Input() isBusy = false;

  @Output() onSubmit = new EventEmitter();
  @Output() onCancel = new EventEmitter();

  id$: Observable<string>;
  isOpen$: Observable<boolean>;

  private _hasSubmitted = false;
  private _subscription: Subscription;

  // Stores the form's initial values, so it can be re-populated
  // after it's reset when submitting.
  private _initialValues: any;

  constructor(private _store: Store) {}

  ngOnInit() {
    this.id$ = this._store.pipe(select(selectors.selectId));
    this.isOpen$ = this._store.pipe(select(selectors.selectIsOpen));

    this._initialValues = this.form.value;

    // TODO: this is an easy way to reset the form based on actions
    // sent to the store, without having to keep the whole content
    // of the form in the store. Not a very robust solution, so it
    // needs refactoring when more time is available.
    this._subscription = this._store
      .pipe(select(selectors.shouldReset))
      .subscribe((shouldReset: boolean) => {
        if (shouldReset && this.form) {
          this.form.reset();
          this.form.patchValue(this._initialValues);
        }
      });
  }

  ngOnDestroy() {
    this._subscription.unsubscribe();
  }

  handleSubmit() {
    if (this.form.valid) {
      this.onSubmit.emit(this.form.value);
      this._hasSubmitted = true;
    }
  }

  handleCancel() {
    this._store.dispatch(actions.closeModalForm());
    //this.form.reset();
    this.onCancel.emit();
  }

  handleClickOnBackdrop(e: MouseEvent) {
    const target = e.target as HTMLElement;
    if (target.classList.contains('drawer-form')) {
      this.handleCancel();
    }
  }

  // The form doesn't reset after the submit button is pressed, because
  // it might be in a busy state while other services are performing
  // operations. This method tracks when to submit.
  resetIfSubmitted() {
    this._hasSubmitted && this.form.reset();
    this._hasSubmitted = false;
  }
}
