/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * UiMap Component
 *
 * Shows a MapBox map with markers.
 */

import {
  Component,
  Input,
  OnInit,
  OnChanges,
  AfterViewInit,
  ViewEncapsulation,
  SimpleChanges,
  ChangeDetectionStrategy,
  HostListener,
  Output,
  EventEmitter,
  OnDestroy
} from '@angular/core';
import * as mapboxgl from 'mapbox-gl';
import { Router } from '@angular/router';
import { ConfigService } from '@shared/config';
import { IMarker } from './types';

@Component({
  selector: 'ui-map',
  templateUrl: 'ui-map.component.html',
  styleUrls: ['ui-map.component.scss'],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UiMapComponent implements OnInit, OnChanges, AfterViewInit, OnDestroy {
  @Input() name = 'map';
  @Input() zoom = 6;
  @Input() lat = 45.899977;
  @Input() lng = 6.172652;
  @Input() markers: IMarker[] = [];
  @Input() center: IMarker;

  @Output() onHoverMarker = new EventEmitter();

  map: mapboxgl.Map;
  style = 'mapbox://styles/mapbox/streets-v11';

  private _addedMarkers: mapboxgl.Marker[] = [];

  constructor(private _config: ConfigService, private _router: Router) {}

  ngOnInit() {}

  ngOnDestroy(){
    this.map.remove();
  }

  ngAfterViewInit() {
    // Create the map.
    this.map = new mapboxgl.Map({
      accessToken: this._config.get('mapAPIKey'),
      container: this.name,
      style: 'mapbox://styles/mapbox/light-v10',
      zoom: this.zoom,
      center: [this.lng, this.lat],
      // TODO: this will remove the copyright box from MapBox.
      // need to review to make sure we are not breaking
      // terms of usage.
      attributionControl: false,
    });

    // Add the buttons for zoom in and out.
    this.map.addControl(
      new mapboxgl.NavigationControl({
        showCompass: false,
        showZoom: true,
      }),
      'bottom-right'
    );

    // Add markers for the first time.
    this.addMarkers(this.markers);

    // Resize for the first time after the map has loaded.
    this.map.on('data', () => this.map.resize());
  }

  ngOnChanges(changes: SimpleChanges) {
    if (this.map) {
      // New markers have been set
      if (changes.hasOwnProperty('markers')) {
        this.clearMarkers();
        this.addMarkers(changes.markers.currentValue);
      }

      // New zoom level has been set
      if (changes.hasOwnProperty('zoom')) {
        this.map.setZoom(changes.zoom.currentValue);
      }

      // A Marker has been set as the center for the map
      if (changes.hasOwnProperty('center')) {
        const marker: IMarker = changes.center.currentValue;
        this.map.flyTo({
          center: [marker.coords.lng, marker.coords.lat],
          zoom: 4,
        });
      }
    }
  }

  clearMarkers() {
    this._addedMarkers.forEach((marker) => marker.remove());
  }

  // Resize the map container when the Window's size changes
  @HostListener('window:resize', ['$event'])
  onResize() {
    this.map.resize();
  }

  // Add points of interest to the map.
  addMarkers(markers: IMarker[]) {
    const router = this._router;

    if (markers && markers.length) {
      let coords: number[][] = [];

      markers.forEach((marker) => {
        const el = document.createElement('div');
        el.classList.add('map__marker');
        el.innerHTML = marker.shortText || '';
        el.title = marker.name;
        el.style.backgroundColor = marker.color || '#ff0000';

        el.addEventListener('click', function () {
          if (marker.url) {
            router.navigate([marker.url]);
          }
        });

        el.addEventListener('mouseenter', () => {
          this.onHoverMarker.emit(marker);
        });

        const markerCoords: mapboxgl.LngLatLike = [
          marker.coords.lng,
          marker.coords.lat,
        ];

        const newMaker = new mapboxgl.Marker({
          element: el,
        })
          .setLngLat(markerCoords)
          .addTo(this.map);

        this._addedMarkers.push(newMaker);

        coords.push(markerCoords);
      });

      // If there is more than one marker, calculate automatically
      // fitting bounds and apply, else, just center on the only
      // marker that has been added to the map.

      if (markers.length > 1) {
        this.map.fitBounds(this.getBounds(coords));
        this.map.resize();
        return;
      }

      this.map.setCenter([markers[0].coords.lng, markers[0].coords.lat]);
    }
  }

  // Generates a boundary box that covers all the markers in the map.
  // TODO: this code is temporal, it needs to be reviewed and refactored.
  // Given an array of coordinates, calculate the smallest box that
  // covers them all with an offset.
  getBounds(points: number[][]): mapboxgl.LngLatBoundsLike {
    const offset = 10;
    let bounds: mapboxgl.LngLatBoundsLike = [100, 100, -100, -100];

    for (let i = 0; i < points.length; i++) {
      if (points[i][0] < bounds[0]) {
        bounds[0] = points[i][0];
      }
      if (points[i][1] < bounds[1]) {
        bounds[1] = points[i][1];
      }
      if (points[i][0] > bounds[2]) {
        bounds[2] = points[i][0];
      }
      if (points[i][1] > bounds[3]) {
        bounds[3] = points[i][1];
      }
    }
    bounds[0] = bounds[0] - offset;
    bounds[1] = bounds[1] - offset;
    bounds[2] = bounds[2] + offset;
    bounds[3] = bounds[3] + offset;

    return bounds;
  }
}
