/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * SiteHttpErrorsInterceptor
 *
 * HTTP Interceptor that handles client and server errors for the
 * whole application.
 */

import {
  HttpErrorResponse,
  HttpEvent,
  HttpHandler,
  HttpInterceptor,
  HttpRequest,
} from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Observable, throwError } from 'rxjs';
import { catchError, retry, take } from 'rxjs/operators';
import { Store, select } from '@ngrx/store';
import {
  selectors as authSelectors,
  actions as authActions,
} from '@auth/state';
import { actions as notificationsActions } from '@shared/notifications';

@Injectable()
export class SiteHttpErrorsInterceptor implements HttpInterceptor {
  constructor(private _store: Store, private _router: Router) {}

  intercept(
    req: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    return next.handle(req).pipe(
      catchError((res: HttpErrorResponse) => {
        let errorMessage = '';

        // server-side error
        switch (res.status) {
          // Connection refused (server down)
          case 0:
            errorMessage =
              'The server is not responding, please check you Internet connection and try again.';
            this.pushErrorNotification(errorMessage);
            break;
          // Unauthorized, user is not logged in or his token has expired (or is malformed)
          case 401:
            // If there is user data in the state, it means that the JWT token has expired.
            // We need to clean the auth state and the token in the local storage
            // before sending the user to the login page.
            // We'll use the logout action from the Auth module.
            this._store
              .pipe(select(authSelectors.selectUser), take(1))
              .subscribe((authData) => {
                errorMessage = res.error.message;
                if (authData) {
                  this._store.dispatch(authActions.logOut({ id: authData.id }));
                  errorMessage =
                    'Your session has expired, please, sign in again to continue using NDT-Link.';
                }
                this.pushErrorNotification(errorMessage);

                // Forward the user to the login page
                this._router.navigate(['/users/login']);
              });

            break;
          // Forbidden, user is logged in but doesn't have enough credentials to access the resouce.
          case 403:
            this.pushErrorNotification(
              'You do not have sufficient privileges to perform this operation. Contact your NDT-Link manager for more information.'
            );
            break;
            break;
          // Too many requests
          case 429:
            errorMessage =
              'You have sent to NDT-Link too many requests in a very short time. For our protection, we are going to block you for a minute.';
            this.pushErrorNotification(errorMessage);
            break;
          // Internal server error
          case 500:
            errorMessage =
              'There was a problem processing your request. Please, try again.';
            this.pushErrorNotification(errorMessage);
            break;
          // Everything else
          default:
            errorMessage = res.error.message;
            this.pushErrorNotification(errorMessage);
        }

        return throwError(errorMessage);
      })
    );
  }

  pushInfoNotification(body: string) {
    this._store.dispatch(
      notificationsActions.push({
        item: { level: 'info', body },
      })
    );
  }

  pushErrorNotification(body: string) {
    this._store.dispatch(
      notificationsActions.push({
        item: { level: 'error', body },
      })
    );
  }
}
