import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  filter,
  map,
  mergeMap,
  withLatestFrom,
} from 'rxjs/operators';
import * as actions from './actions';
import { FEATURE_STATE_NAME } from './reducer';
import { SoftwareModulesService } from '../softwareModules.service';
import {
  SoftwareModuleListDto,
  SoftwareModuleDto,
  SoftwareModuleListItemDto,
} from '../dto';
import { Observable, of } from 'rxjs';

@Injectable()
export class modelEffects {
  constructor(
    private _actions$: Actions,
    private _crudService: SoftwareModulesService,
    private _store: Store
  ) {}

  // Get all items from the backend.
  loadItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItems),
      withLatestFrom(this._store),
      mergeMap(([action, state]) =>
        this._crudService
          .getAll(action.searchText, state[FEATURE_STATE_NAME].sortOrder)
          .pipe(
            map((list: SoftwareModuleListDto) =>
              actions.loadItemsSuccess({ items: list.data })
            ),
            catchError(() => of(actions.loadItemsError()))
          )
      )
    )
  );

  // Get one item from the backend.
  loadItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItem),
      mergeMap((action) =>
        this._crudService.getOne(action.id).pipe(
          map((item: SoftwareModuleDto) => actions.loadItemSuccess({ item })),
          catchError(() => of(actions.loadItemError()))
        )
      )
    )
  );

  // Search items from the backend.
  // Apply debounce measures to prevent reaching the maximum hit rate per
  // minute while the user is typing the search box.
  searchItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.searchItems),
      debounceTime(500),
      map((val) => ({ ...val, searchText: val.searchText.trim() })),
      distinctUntilChanged((prev, next) => prev.searchText === next.searchText),
      map((action) => actions.loadItems({ searchText: action.searchText }))
    )
  );

  // Update the sort order.
  sortItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.sortItems),
      map(() => actions.loadItems({}))
    )
  );

  // Create a new item in the backend.
  addItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItem),
      mergeMap((action) =>
        this._crudService.create(action.item).pipe(
          map((item: SoftwareModuleListItemDto) =>
            actions.addItemSuccess({ item })
          ),
          catchError(() => of(actions.addItemError()))
        )
      )
    )
  );

  // Load the details of the item created in the backend.
  loadAddedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id }))
    )
  );

  // Update an existing item in the backend.
  updateItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItem),
      mergeMap((action) =>
        this._crudService.update(action.id, action.updatedItem).pipe(
          map(({ id, ...changes }: SoftwareModuleListItemDto) =>
            actions.updateItemSuccess({ item: { id, changes } })
          ),
          catchError(() => of(actions.updateItemError()))
        )
      )
    )
  );

  // Load the details of the item that has been updated.
  loadUpdatedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id.toString() }))
    )
  );

  // Delete an item in the backend.
  deleteItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItem),
      mergeMap((action) =>
        this._crudService.delete(action.id).pipe(
          map(() => actions.deleteItemSuccess({ id: action.id })),
          catchError(() => of(actions.deleteItemError()))
        )
      )
    )
  );

  loadForSelector$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadSelectorItems),
      debounceTime(500),
      mergeMap((action) => {
        let cb$: Observable<SoftwareModuleListItemDto[]>;

        switch (action.forModel) {
          case 'system':
            cb$ = this._crudService.getForSystem(action.id);
            break;
          case 'software':
            cb$ = this._crudService.getForSoftware(action.id);
            break;
          case 'product':
            cb$ = this._crudService.getForProduct(action.id);
            break;
        }

        return cb$.pipe(
          map((items: SoftwareModuleListItemDto[]) =>
            actions.loadSelectorItemsSuccess({
              items,
            })
          )
        );
      })
    )
  );

  // Get all items for the given software.
  loadItemsForSoftware$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItemsForSoftware),
      mergeMap((action) =>
        this._crudService
          .getForSoftware(action.softwareId)
          .pipe(
            map((items: SoftwareModuleListItemDto[]) =>
              actions.loadItemsSuccess({ items })
            )
          )
      )
    )
  );
}
