import { Component, Input, OnInit, OnChanges } from '@angular/core';
import { Observable } from 'rxjs';
import { Store, select } from '@ngrx/store';
import { actions, selectors } from '../../state';
import { IAuthUserData, selectors as authSelectors } from '@auth';
import { UserProfileType, UserRoleType, UserInnerspecRoleType } from '@users';
import { actions as modalFormActions } from '@shared/modal-form';
import { actions as modalActions } from '@shared/modal';
import { SubscriptionListItemDto } from '../../dto';

@Component({
  selector: 'subscriptions-table',
  templateUrl: 'subscriptions-table.component.html',
})
export class SubscriptionsTableComponent implements OnInit, OnChanges {
  @Input() systemId: string;

  items$: Observable<SubscriptionListItemDto[]>;
  isLoading$: Observable<boolean>;
  currentUser$: Observable<IAuthUserData>;

  readonly MODAL_CREATE_ID = 'subscriptions_create_form';
  readonly MODAL_PURCHASE_ID = 'subscriptions_purchase_form';
  readonly MODAL_DELETE_CONFIRM_ID = 'subscriptions_delete_confirm';

  constructor(private _store: Store) {}

  ngOnInit() {
    this.items$ = this._store.pipe(select(selectors.selectItems));
    this.isLoading$ = this._store.pipe(select(selectors.selectIsLoadingItems));
    this.currentUser$ = this._store.pipe(select(authSelectors.selectUser));
  }

  ngOnChanges() {
    this._store.dispatch(
      actions.loadItemsForSystem({ systemId: this.systemId })
    );
  }

  showAddNewForm(user: IAuthUserData) {
    if (UserProfileType.user === UserProfileType[user.profile]) {
      this._store.dispatch(
        modalFormActions.openModalForm({ id: this.MODAL_PURCHASE_ID })
      );
    }

    if (UserProfileType.innerspec === UserProfileType[user.profile]) {
      this._store.dispatch(
        modalFormActions.openModalForm({ id: this.MODAL_CREATE_ID })
      );
    }
  }

  selectItem(id: string) {
    this._store.dispatch(actions.selectItem({ id }));
  }

  showConfirmDelete() {
    this._store.dispatch(
      modalActions.openModal({ id: this.MODAL_DELETE_CONFIRM_ID })
    );
  }

  /**
   * Parses if the current user can add/purchase/delete subscriptions.
   * For Innerspec accounts, Manager and Superuser are the only allowed.
   * For User accounts, Manager and Corporate Manager are the only allowed.
   * @param user Curerren logged in user
   * @returns true if user can add/purchase subscriptions
   */
  canChangeSubscription(user: IAuthUserData) {
    if (UserProfileType.user === UserProfileType[user.profile]) {
      return [UserRoleType.coporate_manager, UserRoleType.manager].includes(
        UserRoleType[user.role]
      );
    }

    if (UserProfileType.innerspec === UserProfileType[user.profile]) {
      return [
        UserInnerspecRoleType.manager,
        UserInnerspecRoleType.superuser,
      ].includes(UserInnerspecRoleType[user.role]);
    }

    return false;
  }
}
