import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { catchError, map, mergeMap, withLatestFrom } from 'rxjs/operators';
import * as actions from './actions';
import { FEATURE_STATE_NAME } from './reducer';
import { SubscriptionsService } from '../subscriptions.service';
import {
  SubscriptionListDto,
  SubscriptionDto,
  SubscriptionListItemDto,
  PaymentConfirmationURL,
} from '../dto';
import { of } from 'rxjs';

@Injectable()
export class modelEffects {
  constructor(
    private _actions$: Actions,
    private _crudService: SubscriptionsService,
    private _store: Store
  ) {}

  // Get all items from the backend.
  loadItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItems),
      withLatestFrom(this._store),
      mergeMap(([action, state]) =>
        this._crudService
          .getAll(action.searchText, state[FEATURE_STATE_NAME].sortOrder)
          .pipe(
            map((list: SubscriptionListDto) =>
              actions.loadItemsSuccess({ items: list.data })
            ),
            catchError(() => of(actions.loadItemsError()))
          )
      )
    )
  );

  // Get one item from the backend.
  loadItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItem),
      mergeMap((action) =>
        this._crudService.getOne(action.id).pipe(
          map((item: SubscriptionDto) => actions.loadItemSuccess({ item })),
          catchError(() => of(actions.loadItemError()))
        )
      )
    )
  );

  // Create a new item in the backend.
  addItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItem),
      mergeMap((action) =>
        this._crudService.create(action.item).pipe(
          map((item: SubscriptionListItemDto) =>
            actions.addItemSuccess({ item })
          ),
          catchError(() => of(actions.addItemError()))
        )
      )
    )
  );

  // Purchase a new subscription (for user profiles).
  // Get a confirmation URL from the payment service where the
  // user will be forward to confirm the purchase amount and
  // enter payment method.
  purchaseGetConfirmationURL$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.purchaseGetConfirmationURL),
      mergeMap(({ item, dialogId }) =>
        this._crudService.purchaseGetConfirmationURL(item).pipe(
          map((data: PaymentConfirmationURL) =>
            actions.purchaseGetConfirmationURLSuccess({ data, dialogId })
          ),
          catchError(() => of(actions.purchaseGetConfirmationURLError()))
        )
      )
    )
  );

  // Load the details of the item created in the backend.
  loadAddedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id }))
    )
  );

  // Update an existing item in the backend.
  updateItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItem),
      mergeMap((action) =>
        this._crudService.update(action.id, action.updatedItem).pipe(
          map(({ id, ...changes }: SubscriptionListItemDto) =>
            actions.updateItemSuccess({ item: { id, changes } })
          ),
          catchError(() => of(actions.updateItemError()))
        )
      )
    )
  );

  // Load the details of the item that has been updated.
  loadUpdatedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id.toString() }))
    )
  );

  // Delete an item in the backend.
  deleteItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItem),
      mergeMap((action) =>
        this._crudService.delete(action.id).pipe(
          map(() => actions.deleteItemSuccess({ id: action.id })),
          catchError(() => of(actions.deleteItemError()))
        )
      )
    )
  );

  // Get all items from the given location.
  loadItemsForSystem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItemsForSystem),
      mergeMap((action) =>
        this._crudService
          .getForSystem(action.systemId)
          .pipe(
            map((items: SubscriptionListItemDto[]) =>
              actions.loadItemsSuccess({ items })
            )
          )
      )
    )
  );
}
