import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { catchError, map, mergeMap } from 'rxjs/operators';
import { of } from 'rxjs';
import * as actions from './actions';
import { SystemAccountsService } from '../systemAccounts.service';
import { ModelTypes, SystemAccountDto } from '../dto';

@Injectable()
export class modelEffects {
  constructor(
    private _actions$: Actions,
    private _crudService: SystemAccountsService
  ) {}

  // Get all items from the backend.
  loadItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItems),
      mergeMap((action) =>
        this._crudService.getForSystem(action.systemId).pipe(
          map((items: SystemAccountDto[]) =>
            actions.loadItemsSuccess({ items })
          ),
          catchError(() => of(actions.loadItemsError()))
        )
      )
    )
  );

  // Get one item from the backend.
  loadItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItem),
      mergeMap((action) =>
        this._crudService.getOne(action.id).pipe(
          map((item: SystemAccountDto) => actions.loadItemSuccess({ item })),
          catchError(() => of(actions.loadItemError()))
        )
      )
    )
  );

  // Create a new item in the backend.
  addItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItem),
      mergeMap((action) =>
        this._crudService.create(action.item).pipe(
          map((item: SystemAccountDto) => actions.addItemSuccess({ item })),
          catchError(() => of(actions.addItemError()))
        )
      )
    )
  );

  // Load the details of the item created in the backend.
  loadAddedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id }))
    )
  );

  // Update an existing item in the backend.
  updateItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItem),
      mergeMap((action) =>
        this._crudService.update(action.id, action.updatedItem).pipe(
          map(({ id, ...changes }: SystemAccountDto) =>
            actions.updateItemSuccess({ item: { id, changes } })
          ),
          catchError(() => of(actions.updateItemError()))
        )
      )
    )
  );

  // Load the details of the item that has been updated.
  loadUpdatedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id.toString() }))
    )
  );

  // Delete an item in the backend.
  deleteItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItem),
      mergeMap((action) =>
        this._crudService.delete(action.id).pipe(
          map(() => actions.deleteItemSuccess({ id: action.id })),
          catchError(() => of(actions.deleteItemError()))
        )
      )
    )
  );

  // Get all items from the given model.
  loadItemsForModel$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItemsForModel),
      mergeMap((action) =>
        this._crudService
          .getForModel(action.model, action.id)
          .pipe(
            map((items: SystemAccountDto[]) =>
              actions.loadItemsSuccess({ items })
            )
          )
      )
    )
  );
}
