import {
  Component,
  Input,
  OnInit,
  OnChanges,
  OnDestroy,
  SimpleChanges,
  ChangeDetectionStrategy,
} from '@angular/core';
import { Observable } from 'rxjs';
import { tap } from 'rxjs/operators';
import { Store, select } from '@ngrx/store';
import { actions, selectors } from '../../state';
import { SystemStatusDto, StatusModeType } from '../../dto';
import { IMarker } from '@shared/ui/components/ui-map';

// The type 'global' will make the map to show markers for
// the last status for ALL systems in NDT-Link or users's
// organization.
// The type 'history' will make the map to show markers for
// the last status updated for one specific system, whose
// id myst be passed as input to the component.
export type SystemStatusMapType = 'global' | 'history';

@Component({
  selector: 'systemStatus-map',
  templateUrl: 'systemStatus-map.component.html',
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SystemStatusMapComponent implements OnInit, OnChanges, OnDestroy {
  @Input() name = 'system-status-map';
  @Input() type: SystemStatusMapType = 'global';
  @Input() systemId: string;

  items$: Observable<SystemStatusDto[]>;
  isLoading$: Observable<boolean>;
  markers: IMarker[] = [];

  timer: any;

  colors = {
    [StatusModeType.error]: 'red',
    [StatusModeType.auto]: 'green',
    [StatusModeType.inspecting]: 'green',
    [StatusModeType.idle]: 'yellow',
  };

  constructor(private _store: Store) {}

  ngOnInit() {
    this.items$ = this._store.pipe(select(selectors.selectItems)).pipe(
      // Parse the items' content to create the markers for the map.
      tap((items) => {
        this.markers = items.map((item) => ({
          name: `${item.systemSN} - ${item.productName}`,
          shortText: item.mode === StatusModeType.error ? '!!' : '',
          url: `/systems/${item.systemId}`,
          color: this.colors[StatusModeType[item.mode]],
          coords: {
            lat: parseFloat(item.lat),
            lng: parseFloat(item.lng),
          },
        }));
      })
    );
    this.isLoading$ = this._store.pipe(select(selectors.selectIsLoadingItems));

    if (this.type === 'global') {
      const date = new Date().toJSON().slice(0, 10);

      this.timer = setInterval(
        () => this._store.dispatch(actions.loadItemsForAllSystems({ date })),
        10000
      );
    }
  }

  ngOnChanges(changes: SimpleChanges) {
    if (this.type === 'history' && changes.hasOwnProperty('systemId')) {
      const date = new Date().toJSON().slice(0, 10);

      this._store.dispatch(
        actions.loadItems({
          systemId: changes.systemId.currentValue,
          date,
        })
      );
    }
  }

  ngOnDestroy() {
    clearInterval(this.timer);
  }
}
