/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Users Edit Component
 */

import { Component, Input, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { tap } from 'rxjs/operators';
import { Store, select } from '@ngrx/store';
import { FormBuilder, Validators } from '@angular/forms';
import { FormComponent } from '@shared/classes/FormComponent';
import { actions, selectors } from '../../state';
import {
  UserProfileType,
  UserInnerspecRoleType,
  UserRoleType,
  UserDto,
  UserWriteDto,
} from '../../dto';
import { baseForm } from '../baseForm';
import * as uiValidators from '@shared/ui/validators';

@Component({
  selector: 'users-edit',
  templateUrl: 'users-edit.component.html',
})
export class UsersEditComponent extends FormComponent implements OnInit {
  @Input() id: string;
  @Input() isInnerspec: boolean;

  isSaving$: Observable<boolean>;
  item$: Observable<UserDto>;

  profileTypes: any[];
  roleTypes: any[];

  tabItems = [
    { id: 'general', label: 'General' },
    { id: 'contact_information', label: 'Contact' },
    { id: 'account', label: 'Account' },
    { id: 'other', label: 'Other' },
  ];

  constructor(private _formBuilder: FormBuilder, private _store: Store) {
    super();
  }

  ngOnInit() {
    this.isSaving$ = this._store.pipe(select(selectors.selectIsSavingItem));

    // Load the item to edit.
    // We pipe in to patch the form with the values received from the selector.
    this.item$ = this._store.pipe(select(selectors.selectLoadedItem)).pipe(
      tap((values) => {
        this.handleProfileChange(values?.profile);

        this.form.patchValue({
          ...values,
          location: { id: values?.locationId, name: values?.locationName },
        });
      })
    );

    // Initialize the form.
    this.form = this._formBuilder.group({
      ...baseForm,
      password: [, uiValidators.password()],
      confirmPassword: [, uiValidators.equalTo('password')],
      name: [, [Validators.minLength(3), Validators.maxLength(60)]],
      title: [, Validators.maxLength(60)],
      department: [, Validators.maxLength(60)],
      phone: [
        ,
        Validators.pattern(
          /(\+\d{1,3}\s?)?((\(\d{3}\)\s?)|(\d{3})(\s|-?))(\d{3}(\s|-?))(\d{4})(\s?(([E|e]xt[:|.|]?)|x|X)(\s?\d+))?/
        ),
      ],
      file: [],
      avatar: [],
      description: [],
      notes: [],
    });
  }

  handleProfileChange(profile: string) {
    const isInnerspec = UserProfileType[profile] === UserProfileType.innerspec;
    const types = isInnerspec ? UserInnerspecRoleType : UserRoleType;
    this.roleTypes = this.enumToSelectItems(types);
  }

  handleSubmit(id: string, values: any) {
    // We need to flatten the selector before sending to backend.
    const updatedItem = { ...values, location: values.location.id };

    this._store.dispatch(actions.updateItem({ id, updatedItem }));
  }
}
