/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Redux model Effects for Users.
 */

import { Injectable } from '@angular/core';
import { Store } from '@ngrx/store';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  filter,
  map,
  mergeMap,
  withLatestFrom,
} from 'rxjs/operators';
import { of } from 'rxjs';
import * as actions from './actions';
import { FEATURE_STATE_NAME } from './reducer';
import { UsersService } from '../users.service';
import { UsersListDto, UserDto, UserListItemDto } from '../dto';

@Injectable()
export class modelEffects {
  constructor(
    private _actions$: Actions,
    private _crudService: UsersService,
    private _store: Store
  ) {}

  // Get all items from the backend.
  loadItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItems),
      withLatestFrom(this._store),
      mergeMap(([action, state]) =>
        this._crudService
          .getAll(action.searchText, state[FEATURE_STATE_NAME].sortOrder)
          .pipe(
            map((list: UsersListDto) =>
              actions.loadItemsSuccess({ items: list.data })
            ),
            catchError(() => of(actions.loadItemsError()))
          )
      )
    )
  );

  // Get one item from the backend.
  loadItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItem),
      mergeMap((action) =>
        this._crudService.getOne(action.id).pipe(
          map((item: UserDto) => actions.loadItemSuccess({ item })),
          catchError(() => of(actions.loadItemError()))
        )
      )
    )
  );

  // Search items from the backend.
  // Apply debounce measures to prevent reaching the maximum hit rate per
  // minute while the user is typing the search box.
  searchItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.searchItems),
      debounceTime(500),
      map((val) => ({ ...val, searchText: val.searchText.trim() })),
      distinctUntilChanged((prev, next) => prev.searchText === next.searchText),
      map((action) => actions.loadItems({ searchText: action.searchText }))
    )
  );

  // Update the sort order.
  sortItems$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.sortItems),
      map(() => actions.loadItems({}))
    )
  );

  // Create a new item in the backend.
  addItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItem),
      mergeMap((action) =>
        this._crudService.create(action.item).pipe(
          map((item: UserListItemDto) => actions.addItemSuccess({ item })),
          catchError(() => of(actions.addItemError()))
        )
      )
    )
  );

  // Load the details of the item created in the backend.
  loadAddedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id }))
    )
  );

  // Update an existing item in the backend.
  updateItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItem),
      mergeMap((action) =>
        this._crudService.update(action.id, action.updatedItem).pipe(
          map(({ id, ...changes }: UserListItemDto) =>
            actions.updateItemSuccess({ item: { id, changes } })
          ),
          catchError(() => of(actions.updateItemError()))
        )
      )
    )
  );

  // Load the details of the item that has been updated.
  loadUpdatedItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.updateItemSuccess),
      map((action) => actions.loadItem({ id: action.item.id.toString() }))
    )
  );

  // Delete an item in the backend.
  deleteItem$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItem),
      mergeMap((action) =>
        this._crudService.delete(action.id).pipe(
          map(() => actions.deleteItemSuccess({ id: action.id })),
          catchError(() => of(actions.deleteItemError()))
        )
      )
    )
  );

  // Reset password.
  resetPassword$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.resetPassword),
      mergeMap((action) =>
        this._crudService.resetPassword(action.email).pipe(
          map(() => actions.resetPasswordSuccess()),
          catchError(() => of(actions.resetPasswordError()))
        )
      )
    )
  );

  // Get all items from the given model.
  loadItemsForModel$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadItemsForModel),
      mergeMap((action) =>
        this._crudService
          .getForModel(action.model, action.id)
          .pipe(
            map((items: UserListItemDto[]) =>
              actions.loadItemsSuccess({ items })
            )
          )
      )
    )
  );

  // Search items from the backend for select boxes.
  // Do not emit if:
  // 1. The user doesn't stop stop typing for at least 500ms.
  // 2. The text to search is not the same as the previous emission.
  // 3. The text to search is not empty.
  searchForSelection$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.searchForSelection),
      debounceTime(500),
      map((val) => ({ ...val, searchText: val.searchText.trim() })),
      filter((val) => val.searchText !== ''),
      distinctUntilChanged((prev, next) => prev.searchText === next.searchText),
      mergeMap((action) =>
        this._crudService.getAllSimple(action.searchText).pipe(
          map((list: UsersListDto) =>
            actions.searchForSelectionSuccess({
              users: list.data,
            })
          )
        )
      )
    )
  );

  loadForSelector$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.loadSelectorItems),
      debounceTime(500),
      mergeMap((action) =>
        this._crudService.getForModel(action.forModel, action.id).pipe(
          map((items: UserListItemDto[]) =>
            actions.loadSelectorItemsSuccess({
              items,
            })
          )
        )
      )
    )
  );
}
