/**
 * NDT-Link Client.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Redux UI Effects for Users.
 */

import { Injectable } from '@angular/core';
import { createEffect, Actions, ofType } from '@ngrx/effects';
import { map, tap } from 'rxjs/operators';
import { Router } from '@angular/router';
import * as actions from './actions';
import { actions as modalFormActions } from '@shared/modal-form';
import { actions as modalActions } from '@shared/modal';
import { actions as notificationActions } from '@shared/notifications';

@Injectable()
export class uiEffects {
  constructor(private _actions$: Actions, private _router: Router) {}

  setModalFormBusy$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItem, actions.updateItem, actions.deleteItem),
      map(() => modalFormActions.setModalFormBusy({ busy: true }))
    )
  );

  setModalFormNotBusy$ = createEffect(() =>
    this._actions$.pipe(
      ofType(
        actions.addItemSuccess,
        actions.updateItemSuccess,
        actions.deleteItemSuccess
      ),
      map(() => modalFormActions.setModalFormBusy({ busy: false }))
    )
  );

  setModalBusy$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItem),
      map(() => modalActions.setBusy({ busy: true }))
    )
  );

  setModalNotBusy$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItemSuccess),
      map(() => modalActions.setBusy({ busy: false }))
    )
  );

  closeModalForms$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItemSuccess, actions.updateItemSuccess),
      map(() => modalFormActions.closeModalForm())
    )
  );

  closeModals$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.deleteItemSuccess, actions.deleteItemError),
      map(() => modalActions.closeModal())
    )
  );

  resetModalForm$ = createEffect(() =>
    this._actions$.pipe(
      ofType(actions.addItemSuccess, actions.updateItemSuccess),
      map(() => modalFormActions.resetModalForm())
    )
  );

  onResetPassword$ = createEffect(
    () =>
      this._actions$.pipe(
        ofType(actions.resetPasswordSuccess),
        tap(() => this._router.navigate(['/users/login']))
      ),
    { dispatch: false }
  );

  pushSuccessNotifications$ = createEffect(() =>
    this._actions$.pipe(
      ofType(
        actions.addItemSuccess,
        actions.updateItemSuccess,
        actions.deleteItemSuccess,
        actions.resetPasswordSuccess
      ),
      map((action) => {
        let body = '';

        switch (action.type) {
          case actions.addItemSuccess.type:
            body = `User ${action.item.email} has been registered! A confirmation email with credentials has been sent to the user.`;
            break;
          case actions.updateItemSuccess.type:
            body = `User ${action.item.changes.email} updated!`;
            break;
          case actions.deleteItemSuccess.type:
            body = 'User deleted!';
            break;
          case actions.resetPasswordSuccess.type:
            body =
              'We have reset your password and sent you an email with further instructions.';
            break;
        }

        return notificationActions.push({ item: { level: 'info', body } });
      })
    )
  );
}
