/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Auth Service
 */

import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { User, System, Session } from '@shared/domain';
import { AuthUserDataDto, AuthSystemDataDto } from './auth.dto';

@Injectable()
export class AuthService {
  constructor(
    @InjectRepository(User) private usersRepository: Repository<User>,
    @InjectRepository(System) private systemsRepository: Repository<System>,
    @InjectRepository(Session) private sessionsRepository: Repository<Session>,

    private jwtService: JwtService,
  ) {}

  async validateUser(email: string, password: string): Promise<User> {
    const foundUser = await this.usersRepository.findOne({ email });
    if (!foundUser) {
      throw new HttpException(
        `User with email ${email} was not found.`,
        HttpStatus.NOT_FOUND,
      );
    }

    const passwordValid = await foundUser.checkPassword(password);
    if (!passwordValid) {
      throw new HttpException(
        `The password provided for user ${email} is not correct.`,
        HttpStatus.UNAUTHORIZED,
      );
    }

    return foundUser;
  }

  async validateSystem(tag: string, password: string): Promise<System> {
    const foundSystem = await this.systemsRepository.findOne({ tag });
    if (!foundSystem) {
      throw new HttpException(
        `System with Asset Tag ${tag} was not found.`,
        HttpStatus.NOT_FOUND,
      );
    }

    const passwordValid = await foundSystem.checkPassword(password);
    if (!passwordValid) {
      throw new HttpException(
        `The password provided for system with Asset Tag ${tag} is not correct.`,
        HttpStatus.FORBIDDEN,
      );
    }

    return foundSystem;
  }

  async login(authUserData: AuthUserDataDto) {
    const token = this.jwtService.sign({
      id: authUserData.id,
      email: authUserData.email,
      name: authUserData.name,
      profile: authUserData.profile,
      role: authUserData.role,
      companyId: authUserData.companyId,
      locationId: authUserData.locationId,
    });

    const newSession = new Session();
    newSession.token = token;
    newSession.userId = authUserData.id;

    await this.sessionsRepository.save(newSession);

    return { token };
  }

  async loginSystem(data: AuthSystemDataDto) {
    const {
      id,
      tag,
      sn,
      pn,
      name,
      type,
      po,
      poDate,
      locationName,
      companyName,
    } = data;
    const payload = {
      id,
      tag,
    };

    return {
      token: this.jwtService.sign(payload),
      sn,
      pn,
      name,
      type,
      po,
      poDate,
      locationName,
      companyName,
    };
  }

  async logout(userId: string) {
    return await this.sessionsRepository.delete({ userId });
  }
}
