/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Auth Guard
 */

import { Injectable, CanActivate, ExecutionContext } from '@nestjs/common';
import { Reflector } from '@nestjs/core';
import { Observable } from 'rxjs';
import { AuthUserDataDto } from '../auth.dto';

@Injectable()
export class AuthGuard implements CanActivate {
  constructor(private reflector: Reflector) {}

  canActivate(
    context: ExecutionContext,
  ): boolean | Promise<boolean> | Observable<boolean> {
    // Get user from the context.
    const userToken: any = context.switchToHttp().getRequest().user;

    // If the user in the context is not a person (i.e. system),
    // we block the navigation for security reasons, instead of
    // letting it go.
    // The data for the user comes from decoding the token and won't
    // match the class AuthUserData with instanceof, so we need to
    // do a manual check (only people will have a profile and role,
    // systems won't).
    if (
      !userToken.hasOwnProperty('profile') ||
      !userToken.hasOwnProperty('role')
    ) {
      return false;
    }

    // It's a person user, cast the type from this point on.
    const user = userToken as AuthUserDataDto;

    // Extract the metadata from the context.
    // The metadata can contain profiles and/or roles, the user
    // must have all of them.
    const profiles = this.reflector.get<string[]>(
      'profiles',
      context.getHandler(),
    );
    const roles = this.reflector.get<string[]>('roles', context.getHandler());

    // If the metadata has profiles, the user must have them.
    if (profiles) {
      if (!profiles.includes(user.profile)) {
        return false;
      }
    }

    // If the metadata had roles, the user must have them.
    if (roles) {
      if (!roles.includes(user.role)) {
        return false;
      }
    }

    // The user satisfies all the conditions, we let the navigation
    // to continue.
    return true;
  }
}
