/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * DTOs for Companies.
 */

import { Expose, Exclude, Type } from 'class-transformer';
import {
  IsNotEmpty,
  IsOptional,
  Length,
  MaxLength,
  IsUrl,
  IsEnum,
} from 'class-validator';
import { IsNotBlank } from '@shared/validators';
import { CompanyType, CompanyIndustry } from '@shared/domain';

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class CompanyListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => CompanyListItemDto)
  data: CompanyListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class CompanyListItemDto {
  @Expose()
  id: string;

  @Expose()
  name: string;

  @Expose()
  type: string;

  @Expose()
  industry: string;

  @Expose()
  logo: string;

  @Expose()
  isActive: boolean;

  @Expose()
  totalLocations: number;

  @Expose()
  totalUsers: number;

  @Expose()
  totalSystems: number;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class CompanyDto {
  @Expose()
  id: string;

  @Expose()
  authorId: string;
  @Expose()
  authorName: string;

  @Expose()
  name: string;

  @Expose()
  description: string;

  @Expose()
  type: string;

  @Expose()
  industry: string;

  @Expose()
  web: string;

  @Expose()
  logo: string;

  @Expose()
  notes: string;

  @Expose()
  isActive: boolean;

  @Expose()
  created: string;

  @Expose()
  updated: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class CompanyCreateDto {
  @IsNotBlank({ message: 'The company name cannot be blank' })
  @Length(3, 150)
  name: string;

  @IsNotEmpty()
  @IsEnum(CompanyType)
  type: CompanyType;

  @IsNotEmpty()
  @IsEnum(CompanyIndustry)
  industry: CompanyIndustry;

  @IsOptional()
  @MaxLength(250)
  description: string;

  @IsOptional()
  logo: string;

  @IsOptional()
  @IsUrl({
    protocols: ['http', 'https'],
    require_valid_protocol: true,
  })
  web: string;

  @IsOptional()
  isActive: boolean;

  @IsOptional()
  notes: string;
}
