/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Companies Module
 */

import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { join } from 'path';
import * as fs from 'fs';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { Company } from '@shared/domain';
import { CompanyDto, CompanyCreateDto, CompanyListDto } from './companies.dto';

@Injectable()
export class CompaniesService {
  constructor(@InjectRepository(Company) public repo: Repository<Company>) {}

  async find(search: string, order: string): Promise<CompanyListDto> {
    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      `
      SELECT 
      company.*, 
      COUNT(location.id) AS totalLocations, COUNT(user.id) AS totalUsers, COUNT(\`system\`.id) AS totalSystems
      FROM company
      LEFT JOIN location ON location.companyId = company.id
      LEFT JOIN user ON user.locationId = location.id
      LEFT JOIN \`system\` ON \`system\`.locationId = location.id
      WHERE company.name LIKE ?
      GROUP BY company.id, company.name
      ORDER BY company.name ${order}
      `,
      [s, order],
    );
    return plainToClass(CompanyListDto, { total: items.length, data: items });
  }

  async findOne(id: string): Promise<CompanyDto> {
    const items: any = await this.repo.query(
      `
      SELECT 
      company.*, 
      COUNT(location.id) AS totalLocations, COUNT(user.id) AS totalUsers, COUNT(system.id) AS totalSystems,
      if(author.name is null, author.email, author.name) AS authorName
      FROM company
      LEFT JOIN location ON location.companyId = company.id
      LEFT JOIN user ON user.locationId = location.id
      LEFT JOIN \`system\` ON \`system\`.locationId = system.id
      LEFT JOIN user AS author ON author.id = company.authorId
      WHERE company.id LIKE ?
      GROUP BY company.id, company.name
      `,
      [id],
    );
    return plainToClass(CompanyDto, items[0]);
  }

  async count(): Promise<number> {
    return this.repo.count();
  }

  async addOne(item: CompanyCreateDto): Promise<CompanyDto> {
    const newItem = await this.repo.save(Object.assign(new Company(), item));
    return this.findOne(newItem.id);
  }

  async updateOne(id: string, changes: any): Promise<CompanyDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      // If the logo has changed, delete the previous one.
      if (foundItem.logo) {
        this.deleteLogo(foundItem);
      }

      const udpatedItem = Object.assign(foundItem, changes);
      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  async deleteOne(id: string): Promise<any> {
    const foundItem = await this.repo.findOne(id);

    if (foundItem) {
      const deleted = await this.repo.delete(id);

      // Prevent deleting system companies
      if (foundItem.isSystem) {
        throw new HttpException(
          'This company cannot be deleted',
          HttpStatus.BAD_REQUEST,
        );
      }

      // TODO: refactor this into an external service to manage files.
      if (deleted && foundItem.logo) {
        this.deleteLogo(foundItem);
      }

      return deleted;
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteLogo(item: Company) {
    const path = join(__dirname, '..', '..', 'upload');
    fs.unlinkSync(`${path}/${item.logo}`);
  }
}
