import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { IssueComment, User, UserProfileType } from '@shared/domain';
import { IssueCommentDto, IssueCommentCreateDto } from './issueComments.dto';
import { HtmlTemplateService } from '@shared/html-template';
import { MailService } from '@shared/mail/mail.service';
import { ConfigService } from '@nestjs/config';

@Injectable()
export class IssueCommentsService {
  baseSelectQuery = `
    SELECT 
      issue_comment.*,
      if(author.name is null, author.email, author.name) AS authorName
    FROM issue_comment
    LEFT JOIN user AS author ON author.id = issue_comment.authorId
  `;

  constructor(
    @InjectRepository(IssueComment) public repo: Repository<IssueComment>,
    @InjectRepository(User) public usersRepo: Repository<User>,
    private _htmlTemplate: HtmlTemplateService,
    private _mail: MailService,
    private _config: ConfigService,
  ) {}

  async find(issueId: string): Promise<IssueCommentDto[]> {
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE issue_comment.issueId = ?
        ORDER BY issue_comment.id ASC
      `,
      [issueId],
    );
    return plainToClass(IssueCommentDto, items);
  }

  async findOne(id: string): Promise<IssueCommentDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE issue_comment.id = ?
      `,
      [id],
    );
    return plainToClass(IssueCommentDto, items[0]);
  }

  async addOne(item: IssueCommentCreateDto): Promise<IssueCommentDto> {
    const newItem = await this.repo.save(
      Object.assign(new IssueComment(), item),
    );
    const savedItem = await this.repo.findOne(newItem.id);

    // TODO: optimize this into a direct query.
    // Send notification email
    const commenter = await this.usersRepo.findOne(newItem.authorId);
    const issue = await savedItem.issue;
    const reporter = await this.usersRepo.findOne(issue.authorId);
    const location = await reporter.location;
    const company = await location.company;

    // Send email to staff
    const issueNumber = issue.number;

    if (commenter.profile === UserProfileType.INNERSPEC) {
      // An Innerspec agent entered the comment.
      // Send the email to the reporter.

      let msg = this._htmlTemplate.toString(
        'issueComments/mail/new_user.html',
        {
          number: issueNumber,
          username: reporter.name || reporter.email,
          agent: commenter.name || commenter.email,
          location: location.name,
          company: company.name,
        },
      );
      this._mail.sendEmail(
        reporter.email,
        `Helpdesk: Issue #${issueNumber} - New Comment`,
        msg,
      );
    }

    if (commenter.profile === UserProfileType.USER) {
      // An Innerspec agent entered the comment.
      // Send the email to the reporter.

      let msg = this._htmlTemplate.toString(
        'issueComments/mail/new_staff.html',
        {
          username: commenter.name || commenter.email,
          number: issueNumber,
          location: location.name,
          company: company.name,
        },
      );
      let sendTo = this._config.get('data.helpdeskEmail');
      this._mail.sendEmail(
        sendTo,
        `Helpdesk: Issue #${issueNumber} - New Comment`,
        msg,
      );
    }

    return this.findOne(savedItem.id);
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }
}
