import { Exclude, Expose, Type } from 'class-transformer';
import { IsEnum, IsNotEmpty, IsOptional, Length } from 'class-validator';
import {
  IssueType,
  IssueStatusType,
  IssuePriorityType,
} from '@shared/domain/issue.entity';

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class IssueListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => IssueListItemDto)
  data: IssueListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class IssueListItemDto {
  @Expose()
  id: string;

  @Expose()
  authorName: string;

  @Expose()
  number: number;

  @Expose()
  subject: string;

  @Expose()
  systemSN: string;

  @Expose()
  productPN: string;
  @Expose()
  productName: string;

  @Expose()
  agentName: string;

  @Expose()
  locationName: string;
  @Expose()
  companyName: string;

  @Expose()
  type: string;

  @Expose()
  priority: string;

  @Expose()
  status: string;

  @Expose()
  created: string;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class IssueDto {
  @Expose()
  id: string;

  @Expose()
  authorId: string;
  @Expose()
  authorName: string;

  @Expose()
  number: number;

  @Expose()
  subject: string;

  @Expose()
  description: string;

  @Expose()
  type: string;

  @Expose()
  systemId: string;
  @Expose()
  systemSN: string;

  @Expose()
  productPN: string;
  @Expose()
  productName: string;

  @Expose()
  agentId: string;
  @Expose()
  agentName: string;

  @Expose()
  locationId: string;
  @Expose()
  locationName: string;
  @Expose()
  companyName: string;

  @Expose()
  priority: string;

  @Expose()
  status: string;

  @Expose()
  created: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class IssueCreateDto {
  @IsNotEmpty()
  @Length(10, 150)
  subject: string;

  @IsOptional()
  description?: string;

  @IsNotEmpty()
  @IsEnum(IssueType)
  type: string;

  @IsOptional()
  @IsEnum(IssuePriorityType)
  priority: string = IssuePriorityType.NORMAL;

  @IsOptional()
  @IsEnum(IssueStatusType)
  status: string = IssueStatusType.OPEN;

  @IsNotEmpty()
  system: string;

  @IsOptional()
  agent: string;
}

/**
 * DTO for totals based on the status of the issue.
 */
export class IssueTotalsDto {
  open = 0;
  pending = 0;
  closed = 0;
}
