import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { Issue, IssueStatusType, User } from '@shared/domain';
import {
  IssueDto,
  IssueCreateDto,
  IssueListDto,
  IssueListItemDto,
  IssueTotalsDto,
} from './issues.dto';
import { HtmlTemplateService } from '@shared/html-template';
import { MailService } from '@shared/mail/mail.service';
import { ConfigService } from '@nestjs/config';

@Injectable()
export class IssuesService {
  baseSelectQuery = `
    SELECT
      issue.*,
      \`system\`.id AS systemId, \`system\`.sn AS systemSN,
      product.id AS productId, product.pn AS productPN, product.name AS productName,
      location.id AS locationId, location.name AS locationName, company.name AS companyName,
      if(author.name is null, author.email, author.name) AS authorName,
      if(agent.name is null, agent.email, agent.name) AS agentName
    FROM issue
    LEFT JOIN \`system\` ON issue.systemId = \`system\`.id
    LEFT JOIN product ON \`system\`.productId = product.id
    LEFT JOIN location ON \`system\`.locationId = location.id
    LEFT JOIN company ON location.companyId = company.id
    LEFT JOIN user AS author ON author.id = issue.authorId
    LEFT JOIN user AS agent ON agent.id = issue.agentId
  `;

  openStatus = [IssueStatusType.OPEN];

  pendingStatus = [
    IssueStatusType.IN_PROGRESS,
    IssueStatusType.NEED_INFORMATION,
    IssueStatusType.READY_TO_TEST,
    IssueStatusType.FIXED,
  ];

  closedStatus = [IssueStatusType.CLOSED];

  constructor(
    @InjectRepository(Issue) public repo: Repository<Issue>,
    @InjectRepository(User) public usersRepo: Repository<User>,
    private _htmlTemplate: HtmlTemplateService,
    private _mail: MailService,
    private _config: ConfigService,
  ) {}

  async find(search: string, order: string, countOnly: boolean): Promise<any> {
    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE issue.subject LIKE ? OR issue.description LIKE ?
        GROUP BY issue.id
        ORDER BY issue.id ${order}
      `,
      [s, s],
    );

    if (countOnly) {
      return this.countIssues(items);
    }

    return plainToClass(IssueListDto, { total: items.length, data: items });
  }

  async findOne(id: string): Promise<IssueDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE issue.id = ?
        GROUP BY issue.id
      `,
      [id],
    );
    return plainToClass(IssueDto, items[0]);
  }

  async addOne(item: IssueCreateDto): Promise<IssueDto> {
    const newItem = await this.repo.save(Object.assign(new Issue(), item));
    const savedItem = await this.repo.findOne(newItem.id);

    // TODO: optimize with a direct SQL query
    const author = await this.usersRepo.findOne(savedItem.authorId);
    const system = await savedItem.system;
    const product = await system.product;
    const location = await author.location;
    const company = await location.company;

    // Send email to staff
    const issueNumber = savedItem.number;

    let msg = this._htmlTemplate.toString('issues/mail/new_staff.html', {
      number: issueNumber,
      username: author.name || author.email,
      location: location.name,
      company: company.name,
      sn: system.sn,
      product: product.name,
      subject: savedItem.subject,
    });
    let sendTo = this._config.get('data.helpdeskEmail');
    this._mail.sendEmail(
      sendTo,
      `Helpdesk: New Issue #${issueNumber} Reported`,
      msg,
    );

    // Send email to the reporter
    msg = this._htmlTemplate.toString('issues/mail/new_user.html', {
      number: issueNumber,
      name: author.name || 'Hello',
      sn: system.sn,
      product: product.name,
    });
    this._mail.sendEmail(
      author.email,
      `NDT-Link: Issue #${issueNumber} Received`,
      msg,
    );

    return this.findOne(savedItem.id);
  }

  async updateOne(id: string, changes: any): Promise<IssueDto> {
    const foundItem = await this.repo.findOne(id);
    const originalStatus = foundItem.status;
    const issueNumber = foundItem.number;

    const author = await this.usersRepo.findOne(foundItem.authorId);

    if (foundItem) {
      Object.assign(foundItem, changes);
      await this.repo.save(foundItem);

      // Send email if the status has changed
      if (foundItem.status !== originalStatus) {
        // To staff
        let msg = this._htmlTemplate.toString(
          'issues/mail/status_update_staff.html',
          {
            number: issueNumber,
            status: foundItem.status,
          },
        );

        let sendTo = this._config.get('data.helpdeskEmail');
        this._mail.sendEmail(
          sendTo,
          `Helpdesk: Issue #${issueNumber} Status Changed`,
          msg,
        );

        // To user
        msg = this._htmlTemplate.toString(
          'issues/mail/status_update_user.html',
          {
            name: author.name || 'Hello',
            number: issueNumber,
            status: foundItem.status,
          },
        );

        this._mail.sendEmail(
          author.email,
          `NDT-Link: Issue #${issueNumber} Status Changed`,
          msg,
        );
      }

      return this.findOne(foundItem.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }

  async findforDashboard(): Promise<any> {
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      GROUP BY issue.id
      ORDER BY issue.id DESC
      LIMIT 5
      `,
    );
    return plainToClass(IssueListItemDto, items);
  }

  async findForCompany(companyId: string, countOnly: boolean): Promise<any> {
    const items: any[] = await this.repo.query(
      `
        SELECT issue.*,
          if(author.name is null, author.email, author.name) AS authorName,
          location.id AS locationId, location.name AS locationName,
          \`system\`.sn AS systemSN,
          product.name AS productName
        FROM issue
        LEFT JOIN user AS author ON author.id = issue.authorId
        LEFT JOIN \`system\` ON \`system\`.id = issue.systemId
        LEFT JOIN location ON location.id = \`system\`.locationId
        LEFT JOIN company ON company.id = location.companyId
        LEFT JOIN product ON \`system\`.productId = product.id 
        WHERE issue.status IN (?) AND company.id = ?
        ORDER BY issue.id DESC
      `,
      [this.openStatus, companyId],
    );

    if (countOnly) {
      return this.countIssues(items);
    }

    return plainToClass(IssueListItemDto, items);
  }

  async findForLocation(locationId: string, countOnly: boolean): Promise<any> {
    const items: any[] = await this.repo.query(
      `
        SELECT issue.*,
          if(author.name is null, author.email, author.name) AS authorName,
          \`system\`.sn AS systemSN,
          product.name AS productName
        FROM issue
        LEFT JOIN user AS author ON author.id = issue.authorId
        LEFT JOIN \`system\` ON \`system\`.id = issue.systemId
        LEFT JOIN location ON location.id = \`system\`.locationId
        LEFT JOIN product ON \`system\`.productId = product.id 
        WHERE issue.status IN (?) AND location.id = ?
        ORDER BY issue.id DESC
      `,
      [this.openStatus, locationId],
    );

    if (countOnly) {
      return this.countIssues(items);
    }

    return plainToClass(IssueListItemDto, items);
  }

  async findForUser(userId: string, countOnly: boolean): Promise<any> {
    const items: any[] = await this.repo.query(
      `
        SELECT issue.*,
          if(user.name is null, user.email, user.name) AS authorName
        FROM issue
        LEFT JOIN user ON user.id = issue.authorId
        WHERE issue.status IN (?) AND user.id = ?
        ORDER BY issue.id DESC
      `,
      [this.openStatus, userId],
    );

    if (countOnly) {
      return this.countIssues(items);
    }

    return plainToClass(IssueListItemDto, items);
  }

  async findForSystem(systemId: string, countOnly: boolean): Promise<any> {
    const items: any[] = await this.repo.query(
      `
        SELECT issue.*,
          if(user.name is null, user.email, user.name) AS authorName,
          \`system\`.sn AS systemSN,
          product.name AS productName
        FROM issue
        LEFT JOIN user ON user.id = issue.authorId
        LEFT JOIN \`system\` ON \`system\`.id = issue.systemId
        LEFT JOIN product ON \`system\`.productId = product.id 
        WHERE issue.status IN (?) AND issue.systemId = ?
        ORDER BY issue.id DESC
      `,
      [this.openStatus, systemId],
    );

    if (countOnly) {
      return this.countIssues(items);
    }

    return plainToClass(IssueListItemDto, items);
  }

  countIssues(issues: IssueDto[]): IssueTotalsDto {
    const totals = new IssueTotalsDto();

    // TODO: this code is temporal, needs refactor.

    // Using a for here instead of other array operators to improve
    // performance when the number of issues in the database is big.

    if (issues) {
      for (let i = 0; i < issues.length; i++) {
        const issueStatus = issues[i].status as IssueStatusType;

        if (this.openStatus.includes(issueStatus)) {
          totals.open++;
        }

        if (this.pendingStatus.includes(issueStatus)) {
          totals.pending++;
        }

        if (this.closedStatus.includes(issueStatus)) {
          totals.closed++;
        }
      }
    }

    return totals;
  }
}
