/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * DTOs for Locations.
 */

import { Exclude, Expose, Type } from 'class-transformer';
import { IsNotEmpty, IsOptional, Length, IsEnum } from 'class-validator';
import { IsNotBlank } from '@shared/validators';
import { LocationType } from '@shared/domain';

export class LocationSelectDto {}

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class LocationsListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => LocationListItemDto)
  data: LocationListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class LocationListItemDto {
  @Expose()
  id: string;

  @Expose()
  name: string;

  @Expose()
  type: string;

  @Expose()
  city: string;

  @Expose()
  country: string;

  @Expose()
  lat: string;

  @Expose()
  lng: string;

  @Expose()
  isActive: boolean;

  @Expose()
  companyId: string;

  @Expose()
  companyName: string;

  @Expose()
  totalUsers: number;

  @Expose()
  totalSystems: number;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class LocationDto extends LocationListItemDto {
  @Expose()
  authorId: string;
  @Expose()
  authorName: string;

  @Expose()
  isInnerspec: boolean;

  @Expose()
  street: string;

  @Expose()
  street2?: string;

  @Expose()
  city: string;

  @Expose()
  zip: string;

  @Expose()
  state: string;

  @Expose()
  phone: string;

  @Expose()
  description: string;

  @Expose()
  notes: string;

  @Expose()
  created: string;

  @Expose()
  updated: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class LocationCreateDto {
  @IsNotBlank({ message: 'The location name cannot be blank' })
  @Length(2, 100)
  name: string;

  @IsEnum(LocationType)
  type: string;

  @IsNotEmpty()
  country: string;

  @IsOptional()
  street: string;

  @IsOptional()
  street2: string;

  @IsOptional()
  city: string;

  @IsOptional()
  zip: string;

  @IsOptional()
  state: string;

  @IsNotEmpty()
  company: string;
}

export class LocationUpdateDto extends LocationCreateDto {
  @IsOptional()
  street: string;

  @IsOptional()
  street2: string;

  @IsOptional()
  city: string;

  @IsOptional()
  zip: string;

  @IsOptional()
  state: string;

  @IsOptional()
  phone: string;

  @IsOptional()
  notes: string;
}
