import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { Location, Company, User } from '@shared/domain';
import {
  LocationDto,
  LocationCreateDto,
  LocationUpdateDto,
  LocationsListDto,
  LocationListItemDto,
} from './locations.dto';
import { GeocodingService } from '@shared/geocoding/geocoding-service';
import { UserProfileType } from '@shared/domain';
import { REQUEST } from '@nestjs/core';

@Injectable()
export class LocationsService {
  baseSelectQuery = `
    SELECT 
    location.*, COUNT(user.id) AS totalUsers, COUNT(\`system\`.id) AS totalSystems,
    company.name as companyName,
    if(author.name is null, author.email, author.name) AS authorName
    FROM location
    LEFT JOIN user ON user.locationId = location.id
    LEFT JOIN \`system\` ON \`system\`.locationId = location.id
    LEFT JOIN company ON location.companyId = company.id
    LEFT JOIN user AS author ON author.id = location.authorId
  `;

  constructor(
    @Inject(REQUEST) private _request,
    @InjectRepository(Location) public repo: Repository<Location>,
    @InjectRepository(Company) public companyRepo: Repository<Company>,
    @InjectRepository(User) public userRepo: Repository<User>,
    private _geocoding: GeocodingService,
  ) {}

  async find(search: string, order: string): Promise<LocationsListDto> {
    // If the user is not staff, filter our all locations that are
    // not part of the parent company.

    const user = this._request.user;
    const forCompanyWhere =
      user.profile !== UserProfileType.INNERSPEC
        ? `AND location.companyId = '${user.companyId}'`
        : '';

    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE location.name LIKE ? ${forCompanyWhere}
      GROUP BY location.id, location.name
      ORDER BY location.name ${order}
      `,
      [s, order],
    );
    return plainToClass(LocationsListDto, { total: items.length, data: items });
  }

  async findOne(id: string): Promise<LocationDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE location.id LIKE ?
      GROUP BY location.id, location.name
      `,
      [id],
    );
    return plainToClass(LocationDto, items[0]);
  }

  async findForCompany(companyId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE location.companyId = ?
        GROUP BY location.id, location.name
        ORDER BY location.name ASC
      `,
      [companyId],
    );
    return plainToClass(LocationListItemDto, items);
  }

  async count(): Promise<number> {
    return this.repo.count();
  }

  async addOne(item: LocationCreateDto): Promise<LocationDto> {
    const foundCompany = await this.companyRepo.findOne(item.company);
    if (!foundCompany) {
      throw new HttpException('Company not found', HttpStatus.NOT_FOUND);
    }

    // Before saving the new location, we need to geocode its address.
    // When creating a new location, we only need the country. The user
    // can add the address later by editing the location.
    const coords = await this._geocoding
      .codeAddress({
        street: item.street,
        street2: item.street2,
        city: item.city,
        state: item.state,
        zip: item.zip,
        country: item.country,
      })
      .toPromise();

    // Save the new item with the calculated coordinates.
    const newItem = Object.assign(new Location(), item);
    newItem.lng = coords[0];
    newItem.lat = coords[1];
    await this.repo.save(newItem);

    // One problem with TypeORM is that the save method doesn't
    // return the saved entity. We need to find it in the
    // database manually.
    const savedItem = await this.repo.findOne(newItem.id);
    return this.findOne(savedItem.id);
  }

  async updateOne(
    id: string,
    changes: LocationUpdateDto,
  ): Promise<LocationDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      // Before saving the new location, we need to geocode its address.
      const coords = await this._geocoding
        .codeAddress({
          street: changes.street,
          street2: changes.street2,
          city: changes.city,
          state: changes.state,
          zip: changes.zip,
          country: changes.country,
        })
        .toPromise();

      const udpatedItem = Object.assign(foundItem, changes);
      udpatedItem.lng = coords[0];
      udpatedItem.lat = coords[1];

      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }
}
