/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Main entry script of the application.
 */

import { NestFactory } from '@nestjs/core';
import { Logger } from '@nestjs/common';
import { ValidationPipe } from '@nestjs/common';
import { initSwagger } from './swagger';
import * as helmet from 'helmet';
import * as rateLimit from 'express-rate-limit';
import { AppModule } from './app.module';
import { ConfigService } from '@nestjs/config';
import { isProduction } from '@shared/infrastructure';
import { seed } from './seed';
import * as express from 'express';
import { join } from 'path';

async function bootstrap() {
  const app = await NestFactory.create(AppModule);
  const logger = new Logger('NDT-Link');

  const config = app.get(ConfigService);
  const clientUrl = config.get('client.url');
  const url = config.get('site.url');
  const port = config.get('site.port');

  logger.log(`Starting version ${config.get('appVersion')}`);

  // Implement basic security measures.
  app.use(helmet());
  app.enableCors({
    // Only the URL of the front-end will be allowed
    // to send requests and receive responses from the
    // application.
    origin: [clientUrl],
    credentials: true,
    // This is to ignore the OPTIONS preflight requests that
    // the browser will send with each request to check
    // if backend supports CORS.
    preflightContinue: false,
  });

  // Limit the number of requests that the client can
  // send to the application to prevent attacks.
  // Maximum is 120 requests per minute, if the client
  // sends more than that, it will receive a 429 response
  // (too many requests) for any consecutive requests.
  // The limit will be removed after one minute of
  // no requests.
  app.use(
    rateLimit({
      windowMs: 60 * 1000,
      max: 120,
    }),
  );

  // Enable static serving of files (images)
  app.use(express.static(join(__dirname, '..', 'upload')));

  // Enable automatic validation of body content in
  // POST, PATCH and PUT requests.
  app.useGlobalPipes(new ValidationPipe());

  // Enable Swagger API browser only if not in production.
  // TODO: replace Swagger with a better documentation tool.
  if (!isProduction()) {
    initSwagger(app);
  }

  // Populate database in case it's empty (first time the
  // application is started in a server).
  await seed(app);

  // Bring up the web server and start listening for incomming
  // connections.
  await app.listen(port, () => {
    logger.log(`Listening from ${url}:${port}..`);
  });
}

bootstrap();
