/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * DTOs for Products.
 */

import { Expose, Exclude, Type } from 'class-transformer';
import {
  IsNotEmpty,
  IsOptional,
  Length,
  MaxLength,
  IsUrl,
  IsEnum,
} from 'class-validator';
import { ProductType } from '@shared/domain';

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class ProductListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => ProductListItemDto)
  data: ProductListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class ProductListItemDto {
  @Expose()
  id: string;

  @Expose()
  pn: string;

  @Expose()
  name: string;

  @Expose()
  type: string;

  @Expose()
  image: string;

  @Expose()
  isActive: boolean;

  @Expose()
  softwareName: string;

  @Expose()
  totalSystems: number;

  @Expose()
  totalLocations: number;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class ProductDto {
  @Expose()
  id: string;

  @Expose()
  pn: string;

  @Expose()
  name: string;

  @Expose()
  type: string;

  @Expose()
  image: string;

  @Expose()
  description: string;

  @Expose()
  softwareId: string;

  @Expose()
  softwareName: string;

  @Expose()
  notes: string;

  @Expose()
  isActive: boolean;

  @Expose()
  created: string;

  @Expose()
  updated: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class ProductCreateDto {
  @IsNotEmpty()
  @Length(8, 8)
  pn: string;

  @IsNotEmpty()
  name: string;

  @IsNotEmpty()
  @IsEnum(ProductType)
  type: ProductType;

  @IsOptional()
  @MaxLength(250)
  description: string;

  @IsOptional()
  image: string;

  @IsOptional()
  isActive: boolean;

  @IsOptional()
  notes: string;

  @IsNotEmpty()
  software: string;
}
