/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Products Service
 */

import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { join } from 'path';
import * as fs from 'fs';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { Product } from '@shared/domain';
import { ProductDto, ProductCreateDto, ProductListDto } from './products.dto';

@Injectable()
export class ProductsService {
  constructor(@InjectRepository(Product) public repo: Repository<Product>) {}

  async find(search: string, order: string): Promise<ProductListDto> {
    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      `
      SELECT 
      product.*, 
      software.id AS softwareId, 
      software.name AS softwareName,
      COUNT(\`system\`.id) AS totalSystems,
      COUNT(location.id) AS totalLocations
      FROM product
      LEFT JOIN software ON software.id = product.softwareId
      LEFT JOIN \`system\` ON \`system\`.productId = product.id
      LEFT JOIN location ON \`system\`.locationId = location.id
      WHERE product.pn LIKE ? OR product.name LIKE ?
      GROUP BY product.id, product.pn
      ORDER BY product.pn ${order}
      `,
      [s, s, order],
    );
    return plainToClass(ProductListDto, { total: items.length, data: items });
  }

  async findOne(id: string): Promise<ProductDto> {
    const items: any = await this.repo.query(
      `
      SELECT 
      product.*, software.id AS softwareId, software.name AS softwareName
      FROM product
      LEFT JOIN software ON software.id = product.softwareId
      WHERE product.id LIKE ?
      GROUP BY product.id, product.pn
      `,
      [id],
    );
    return plainToClass(ProductDto, items[0]);
  }

  async addOne(item: ProductCreateDto): Promise<ProductDto> {
    const newItem = await this.repo.save(Object.assign(new Product(), item));
    return this.findOne(newItem.id);
  }

  async updateOne(id: string, changes: any): Promise<ProductDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      // If the image has changed, delete the previous one.
      if (foundItem.image) {
        this.deleteImage(foundItem);
      }

      const udpatedItem = Object.assign(foundItem, changes);
      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  async deleteOne(id: string): Promise<any> {
    const foundItem = await this.repo.findOne(id);

    if (foundItem) {
      const deleted = await this.repo.delete(id);

      // TODO: refactor this into an external service to manage files.
      if (deleted && foundItem.image) {
        this.deleteImage(foundItem);
      }

      return deleted;
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteImage(item: Product) {
    const path = join(__dirname, '..', '..', 'upload');
    fs.unlinkSync(`${path}/${item.image}`);
  }
}
