/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Seeder.
 *
 * Populates the database for the first time with minimum data
 * needed to start using the application.
 * It inserts Innerspec as company and all its locations. It
 * also inserts a user with full credentials (superuser).
 *
 * This script is not handling errors at the moment.
 */

import { INestApplication } from '@nestjs/common';
import { Logger } from '@nestjs/common';
import {
  Company,
  CompanyType,
  Location,
  LocationType,
  User,
  UserProfileType,
  UserRoleType,
} from '@shared/domain';

export async function seed(app: INestApplication) {
  const configService = app.get('ConfigService');
  const logger = new Logger('NDT-Link/Seeder');

  // Use the TypeOrm repositories defined in the
  // Persistence module.
  const companiesRepo = app.get('CompanyRepository');
  const locationsRepo = app.get('LocationRepository');
  const usersRepo = app.get('UserRepository');

  // If there are already companies in the database we
  // asume that it's already populated and do nothing.
  const totalCompanies = await companiesRepo.count();
  if (totalCompanies > 0) {
    logger.log('Database populated, no seeding needed.');
    return;
  }

  // Create Innerspec company.
  const innerspec = new Company();
  innerspec.id = configService.get('data.innerspecCompanyId');
  innerspec.name = 'Innerspec Technologies, Inc.';
  innerspec.description = 'World Leader in NDT Technologies';
  innerspec.type = CompanyType.OTHER;
  innerspec.isInnerspec = true;
  innerspec.web = 'https://www.innerspec.com';
  innerspec.isSystem = true;

  const innerspecCompany = await companiesRepo.save(innerspec);

  logger.log('Created company Innerspec.');

  // Create Innerspec locations.
  const locations = [
    {
      name: 'Global Headquarters',
      type: LocationType.OFFICE,
      street: '2950 Perrowville Rd',
      city: 'Forest',
      state: 'VA',
      zip: '245607',
      country: 'US',
      phone: '+1434890890',
    },
    {
      name: 'Innerspec Technologies Europe S.L',
      type: LocationType.OFFICE,
      street: 'Pol. Industrial Prado Concejil',
      street2: 'Calle Sanglas, 13',
      city: 'Loeches',
      state: 'Madrid',
      zip: '28890',
      country: 'ES',
      phone: '+34911337024',
    },
    {
      name: 'Innerspec Technologies China, LLC.',
      type: LocationType.OFFICE,
      street: 'Room 311, Building 6 Yaohua Business Plaza',
      street2: '215 Yaohua Road',
      city: 'Shanghai',
      state: 'Shanghai',
      zip: '245607',
      country: 'CN',
      phone: '+86(0)2158358103',
    },
    {
      name: 'Innerspec Technologies Latam S.A de C.V.',
      type: LocationType.OFFICE,
      street: 'Mariano Escobedo 16',
      city: 'Coatepec',
      state: '91517',
      zip: '245607',
      country: 'MX',
      phone: '+522282034112',
    },
    {
      name: 'Innerspec Technologies UK, Ltd.',
      type: LocationType.OFFICE,
      street: 'Royston Rd, Duxford',
      city: 'Cambridge',
      state: 'Cambridge',
      zip: 'CB22 4QH',
      country: 'GB',
      phone: '+441223919725',
    },
  ];

  const createdLocations = await Promise.all(
    locations.map((location) => {
      const newLocation = new Location();
      newLocation.name = location.name;
      newLocation.type = location.type;
      newLocation.street = location.street;
      newLocation.street2 = location.street2;
      newLocation.city = location.city;
      newLocation.state = location.state;
      newLocation.zip = location.zip;
      newLocation.country = location.country;
      newLocation.phone = location.phone;
      newLocation.company = innerspecCompany;
      newLocation.isInnerspec = true;

      return locationsRepo.save(newLocation);
    }),
  );

  logger.log('Created Innerspec locations.');

  // Create the superuser.
  const superuser = new User();
  superuser.isSystem = true;
  superuser.email = configService.get('site.superEmail');
  superuser.password = configService.get('site.superPassword');
  superuser.profile = UserProfileType.INNERSPEC;
  superuser.role = UserRoleType.SUPERUSER;
  superuser.location = createdLocations[0];

  await usersRepo.save(superuser);

  logger.log('Created superuser.');
}
