import { HttpException, HttpStatus } from '@nestjs/common';
import { plainToClassFromExist } from 'class-transformer';
import { Repository } from 'typeorm';

export interface ICrudService<Model> {
  findAll(filter?: string, fields?: string): Promise<Model[]>;
  findOne(id: string): Promise<Model>;
  addOne(item: Model): Promise<Model>;
  updateOne(id: string, changes: Model): Promise<number>;
  deleteOne(id: string): Promise<number>;
}

export class CrudService<Model> implements ICrudService<Model> {
  private _model: { new (): Model };

  constructor(public repo: Repository<Model>) {}

  findAll(filters?: string, fields?: string): Promise<Model[]> {
    const tableName = this.repo.metadata.givenTableName;
    const qb = this.repo.createQueryBuilder();

    if (fields) {
      const selects = fields.split(',').map((field) => `${tableName}.${field}`);
      qb.select(selects);
    }

    if (filters) {
      filters.split(',').forEach((filter) => {
        const params = filter.split(':');
        qb.andWhere(`${tableName}.${params[0]} = :value`, { value: params[1] });
      });
    }

    return qb.getMany();
  }

  findOne(id: string): Promise<Model> {
    return this.repo.findOne(id);
  }

  addOne(item: any): Promise<Model> {
    const newItem = new this._model();
    const p = plainToClassFromExist(newItem, item);
    return this.repo.save(p);

    // return this.repo.save(item);
    // return this.repo.save(item);
  }

  async updateOne(id: string, changes: any): Promise<any> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      const udpatedItem = Object.assign(foundItem, changes);
      return await this.repo.save(udpatedItem);
    }

    new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }
}
