/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * User Entity
 */

import {
  Entity,
  Column,
  ManyToOne,
  BeforeInsert,
  BeforeUpdate,
  Unique,
  BeforeRemove,
} from 'typeorm';
import * as bcrypt from 'bcrypt';
import { BaseEntity, spaceRemover } from './base.entity';
import { Location } from './location.entity';

export enum UserProfileType {
  USER = 'user',
  INNERSPEC = 'innerspec',
  PRODUCT = 'product',
}

export enum UserRoleType {
  MANAGER = 'manager',
  EMPLOYEE = 'employee',
  CORPORATE = 'corporate',
  CORPORATE_MANAGER = 'corporate_manager',
  SUPERUSER = 'superuser',
}

@Entity()
@Unique(['email'])
export class User extends BaseEntity {
  @Column({
    transformer: spaceRemover(),
  })
  email: string;

  @Column()
  password: string;

  @Column({ nullable: true })
  name: string;

  @Column({ nullable: true })
  title: string;

  @Column({ nullable: true })
  department: string;

  @Column({ nullable: true })
  phone: string;

  @Column({
    type: 'enum',
    enum: UserProfileType,
    default: UserProfileType.USER,
  })
  profile: UserProfileType;

  @Column({ type: 'enum', enum: UserRoleType, default: UserRoleType.EMPLOYEE })
  role: UserRoleType;

  @Column({ nullable: true })
  avatar?: string;

  @ManyToOne(() => Location, (location) => location.users)
  location: Promise<Location>;

  /**
   * Determines if the user is a system account (can't be deleted)
   */
  @Column()
  isSystem: boolean;

  /**
   * Encrypts and sets the password with the given uncrypted value.
   * @param password Text value to encrypt and set.
   */
  @BeforeInsert()
  async setPassword() {
    this.password = await this.hashPassword(this.password);
  }

  @BeforeUpdate()
  async updatePassword() {
    if (this.password) {
      this.password = await this.hashPassword(this.password);
    }
  }

  /**
   * Hashes the given password.
   * @param password Password to hash.
   * @returns Hashed password.
   */
  async hashPassword(password: string): Promise<string> {
    const saltRounds = 10;
    const hashedPassword = await bcrypt.hash(password, saltRounds);
    return hashedPassword;
  }

  /**
   * Compares the given hashed password with the stored one
   * for the current user.
   * @param password Hashed password to compare.
   * @returns True if passwords match.
   */
  async checkPassword(password: string): Promise<boolean> {
    return await bcrypt.compare(password, this.password);
  }
}
