/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * HtmlTemplate Module Service
 */

import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import * as path from 'path';
import { readFileSync } from 'fs';
import { isProduction } from '@shared/infrastructure';

@Injectable()
export class HtmlTemplateService {
  private _baseUrl: string;

  constructor(private _config: ConfigService) {
    this._baseUrl = this._config.get('site.url');
  }

  /**
   * Given the path to an HTML template and an object of values,
   * it returns the parsed HTML (tags in the HTML template are
   * replaced by their correspondent value).
   * @param filePath Path to the HTML template.
   * @param values Object with key/values to be used when parsing
   * the template.
   * @returns Parsed HTML.
   */
  toString(filePath: string, values: any): string {
    const rootFolder = isProduction() ? '' : 'src';
    const contentTpl = readFileSync(
      path.resolve(rootFolder, filePath),
      'utf-8',
    );
    const content = this.parseContent(contentTpl, values);

    return this.parseLayout(content);
  }

  /**
   * Parses the content of the template
   * @param tpl HTML template to use for the content.
   * @param values Object with key/values to use to parse the
   * tempalte.
   * @returns Parsed HTML with the content.
   */
  parseContent(tpl: string, values: any): string {
    // First, add standard parameters that will be available to all emails.
    values['baseUrl'] = this._baseUrl;

    // Parse parameters.
    Object.keys(values).forEach(
      (key) => (tpl = tpl.replace(`{{${key}}}`, values[key])),
    );

    return tpl;
  }

  /**
   * Parses the HTML of the layout that contains the parsed
   * HTML of the content.
   * @param content Parsed HTML.
   * @returns Parsed HTML for the layout.
   */
  parseLayout(content: string): string {
    const commands = content.match(/#\S+#/);
    let layoutName = 'default';

    commands.forEach((command) => {
      const params = command.replace('#', '').split(':');

      switch (params[0]) {
        case 'layout':
          if (params.length > 0) {
            layoutName = params[1];
          }
          break;
        // Add more commands here..
        // Structure of command is #name:value#
      }
      content = content.replace(`${command}`, '');
    });

    const layoutTpl = readFileSync(
      path.resolve(__dirname, `layouts/${layoutName}.html`),
      'utf-8',
    );
    return layoutTpl.replace('{{content}}', content);
  }
}
