import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { REQUEST } from '@nestjs/core';
import { InjectRepository } from '@nestjs/typeorm';
import * as paypal from '@paypal/checkout-server-sdk';
import { Payment, SoftwareModule, System } from '@shared/domain';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { PaymentCompletedDto, PaymentWriteDto } from './payment.dto';

@Injectable()
export class PaymentService {
  private _client: any;
  private _clientUrl: string;

  constructor(
    @Inject(REQUEST) private _request,
    private _config: ConfigService,
    @InjectRepository(Payment) public paymentRepo: Repository<Payment>,
    @InjectRepository(System) public systemRepo: Repository<System>,
    @InjectRepository(SoftwareModule)
    public softwareModuleRepo: Repository<SoftwareModule>,
  ) {
    this._clientUrl = this._config.get('client.url');

    // Configure connector to PayPal.
    let environment = new paypal.core.SandboxEnvironment(
      this._config.get('payment.user'),
      this._config.get('payment.password'),
    );
    this._client = new paypal.core.PayPalHttpClient(environment);
  }

  async createPayment(payment: PaymentWriteDto, returnUrl: string) {
    // Get the current user from the request in order to obtain
    // his email for the transaction.
    const currentUser = this._request.user;

    // Create a payment request
    let request = new paypal.orders.OrdersCreateRequest();

    request.requestBody({
      intent: 'CAPTURE',
      // payer: {
      //   payer_id: currentUser.id,
      //   email_address: currentUser?.email,
      // },
      purchase_units: [
        {
          description: 'New Software Module Subscription - NDT-Link',
          amount: {
            currency_code: payment.currency,
            value: payment.total,
            breakdown: {
              item_total: {
                value: payment.total,
                currency_code: payment.currency,
              },
            },
          },
          items: [
            {
              name: payment.softwareModuleName,
              description: payment.description,
              // sku: payment.softwareModuleId,
              quantity: '1',
              unit_amount: {
                value: payment.total,
                currency_code: payment.currency,
              },
            },
          ],
        },
      ],
      application_context: {
        brand_name: 'NDT-Link',
        shipping_preference: 'NO_SHIPPING',
        return_url: returnUrl,
      },
    });

    // Execute the payment request so we can obtain a confirmation
    // URL where to redirect the user for to enter payment information.
    let response = await this._client.execute(request);

    let links = {};
    response.result.links.forEach(
      ({ href, rel, method }) => (links[rel] = { href, method }),
    );

    // Create a new payment with the token received from PayPal.
    // This payment will be process later in the executePayment.
    const token = response.result.id;

    const foundSystem = await this.systemRepo.findOne(payment.systemId);
    if (!foundSystem) {
      throw new HttpException(
        `System with ID ${payment.systemId} not found`,
        HttpStatus.NOT_FOUND,
      );
    }

    const foundSoftwareModule = await this.softwareModuleRepo.findOne(
      payment.softwareModuleId,
    );
    if (!foundSoftwareModule) {
      throw new HttpException(
        `Software Module with ID ${payment.softwareModuleId} not found`,
        HttpStatus.NOT_FOUND,
      );
    }

    const newPayment = new Payment();
    newPayment.token = token;
    newPayment.system = foundSystem;
    newPayment.softwareModule = foundSoftwareModule;
    newPayment.email = '';

    await this.paymentRepo.save(newPayment);

    // Return the confirmation URL to the client
    if (links.hasOwnProperty('approve')) {
      return links['approve'];
    }

    throw new HttpException(
      'The payment request confirmation was not approved by Paypal',
      HttpStatus.INTERNAL_SERVER_ERROR,
    );
  }

  async executePayment(token: string) {
    const request = new paypal.orders.OrdersCaptureRequest(token);
    request.requestBody({});
    // Call API with your client and get a response for your call
    let response = await this._client.execute(request);

    if (response.result) {
      const { status, payer } = response.result;

      if (status === 'COMPLETED') {
        // The payment has been successfull.
        // Add the email of the user who paied
        const foundPayment = await this.paymentRepo.findOne({
          where: { token },
        });

        foundPayment.email = payer.email_address;
        foundPayment.payerId = payer.payer_id;
        foundPayment.completed = true;

        await this.paymentRepo.save(foundPayment);

        return foundPayment.id;
      }
    }

    throw new HttpException(
      `Something went wrong while Paypal was processing the payment`,
      HttpStatus.SERVICE_UNAVAILABLE,
    );
  }
}
