/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * PersistenceHttpErrorFilter
 *
 * Exception filter that sends properly formatted JSON responses to
 * exceptions occured when queries sent to the database through TypeORM fail.
 */

import {
  ExceptionFilter,
  Catch,
  ArgumentsHost,
  HttpStatus,
} from '@nestjs/common';
import { QueryFailedError } from 'typeorm';

@Catch(QueryFailedError)
export class PersistenceHttpErrorFilter implements ExceptionFilter {
  catch(e: any, host: ArgumentsHost) {
    const ctx = host.switchToHttp();
    const response = ctx.getResponse();

    let code = HttpStatus.INTERNAL_SERVER_ERROR;
    let message = `Error code "${e.code}" occured when processing your request. Please, try again and if the problem persists, contact our Customer Support team.`;

    switch (e.code) {
      // A unique column is violated.
      case 'ER_DUP_ENTRY':
        code = HttpStatus.CONFLICT;
        message =
          'Duplicate entry, there is already an item in the database with the same unique value/s.';
        break;
      // Delete fails due to foreign relation constraint.
      case 'ER_ROW_IS_REFERENCED_2':
        code = HttpStatus.CONFLICT;
        message =
          'The item cannot be deleted because there are other items related to it that need to be deleted first.';
        break;
    }

    response.status(code).json({
      message,
    });
  }
}
