/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Software DTOs
 */

import { Exclude, Expose, Type } from 'class-transformer';
import { IsEnum, IsNotEmpty, IsOptional, Length } from 'class-validator';
import { SoftwarePlatformType } from '@shared/domain';

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class SoftwareListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => SoftwareListItemDto)
  data: SoftwareListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class SoftwareListItemDto {
  @Expose()
  id: string;

  @Expose()
  pn: string;

  @Expose()
  name: string;

  @Expose()
  type: string;

  @Expose()
  totalProducts: number;

  @Expose()
  totalSoftwareModules: number;

  @Expose()
  totalSystems: number;

  @Expose()
  isActive: string;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class SoftwareDto {
  @Expose()
  id: string;

  @Expose()
  pn: string;

  @Expose()
  name: string;

  @Expose()
  type: string;

  @Expose()
  description: string;

  @Expose()
  isActive: string;

  @Expose()
  created: string;

  @Expose()
  updated: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class SoftwareCreateDto {
  @IsNotEmpty()
  @Length(8, 8)
  pn: string;

  @IsNotEmpty()
  @IsEnum(SoftwarePlatformType)
  type: SoftwarePlatformType;

  @IsNotEmpty()
  name: string;

  @IsOptional()
  description: string;
}

export class SoftwareUpdateDto extends SoftwareCreateDto {
  @IsOptional()
  notes?: string;
}
