/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * DTOs for SoftwareModules.
 */

import { Exclude, Expose, Transform, Type } from 'class-transformer';
import {
  IsNotEmpty,
  IsNumber,
  IsOptional,
  Length,
  MaxLength,
} from 'class-validator';

export class SoftwareModuleSelectDto {}

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class SoftwareModulesListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => SoftwareModuleListItemDto)
  data: SoftwareModuleListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class SoftwareModuleListItemDto {
  @Expose()
  id: string;

  @Expose()
  pn: string;

  @Expose()
  name: string;

  @Expose()
  @Transform(({ value }) => Boolean(value))
  isFree: boolean;

  @Expose()
  daily: number;

  @Expose()
  weekly: number;

  @Expose()
  monthly: number;

  @Expose()
  annual: number;

  @Expose()
  permanent: number;

  @Expose()
  description: string;

  @Expose()
  isActive: boolean;

  @Expose()
  softwareId: string;

  @Expose()
  softwareName: string;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class SoftwareModuleDto extends SoftwareModuleListItemDto {
  @Expose()
  notes: string;

  @Expose()
  created: string;

  @Expose()
  updated: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class SoftwareModuleCreateDto {
  @IsNotEmpty()
  @Length(8, 8)
  pn: string;

  @IsNotEmpty()
  @Length(2, 100)
  name: string;

  @IsNotEmpty()
  isFree: boolean;

  @IsNotEmpty()
  @IsNumber()
  daily: number;

  @IsNotEmpty()
  @IsNumber()
  weekly: number;

  @IsNotEmpty()
  @IsNumber()
  monthly: number;

  @IsNotEmpty()
  @IsNumber()
  annual: number;

  @IsNotEmpty()
  @IsNumber()
  permanent: number;

  @IsOptional()
  @MaxLength(150)
  description?: string;

  @IsOptional()
  notes?: string;

  @IsNotEmpty()
  software: string;
}
