import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { SoftwareModule, Software } from '@shared/domain';
import {
  SoftwareModuleDto,
  SoftwareModuleCreateDto,
  SoftwareModulesListDto,
  SoftwareModuleListItemDto,
} from './softwareModules.dto';

@Injectable()
export class SoftwareModulesService {
  baseSelectQuery = `
    SELECT 
    software_module.*, 
    software.id as softwareId, software.name as softwareName
    FROM software_module
    LEFT JOIN software ON software_module.softwareId = software.id
  `;

  constructor(
    @InjectRepository(SoftwareModule) public repo: Repository<SoftwareModule>,
    @InjectRepository(Software) public softwareRepo: Repository<Software>,
  ) {}

  async find(search: string, order: string): Promise<SoftwareModulesListDto> {
    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE software_module.pn LIKE ? OR software_module.name LIKE ?
      GROUP BY software_module.id, software_module.pn
      ORDER BY software_module.pn ${order}
      `,
      [s, s, order],
    );
    return plainToClass(SoftwareModulesListDto, {
      total: items.length,
      data: items,
    });
  }

  async findOne(id: string): Promise<SoftwareModuleDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE software_module.id LIKE ?
      GROUP BY software_module.id, software_module.name
      `,
      [id],
    );
    return plainToClass(SoftwareModuleDto, items[0]);
  }

  async findForProduct(productId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        LEFT JOIN product ON product.softwareId = software.id
        WHERE product.id = ?
        GROUP BY software_module.id, software_module.pn
        ORDER BY software_module.name ASC
      `,
      [productId],
    );
    return plainToClass(SoftwareModuleListItemDto, items);
  }

  async findForSoftware(softwareId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE software_module.softwareId = ?
        GROUP BY software_module.id, software_module.pn
        ORDER BY software_module.pn ASC
      `,
      [softwareId],
    );
    return plainToClass(SoftwareModuleListItemDto, items);
  }

  async findForSystem(systemId: string): Promise<any> {
    const items: any = await this.repo.query(
      `
        SELECT
          software_module.*, 
          product.name AS productName,
          software.id AS softwareId, software.name AS softwareName
        FROM software_module
        LEFT JOIN software ON software_module.softwareId = software.id
        LEFT JOIN product ON product.softwareId = software.id
        LEFT JOIN \`system\` ON \`system\`.productId = product.id
        WHERE \`system\`.id = ? AND software_module.isFree = false
        GROUP BY software_module.id, software_module.pn
        ORDER BY software_module.id ASC
      `,
      [systemId],
    );
    return plainToClass(SoftwareModuleListItemDto, items);
  }

  async addOne(item: SoftwareModuleCreateDto): Promise<SoftwareModuleDto> {
    const foundSoftware = await this.softwareRepo.findOne(item.software);

    if (foundSoftware) {
      const newItem = await this.repo.save(
        Object.assign(new SoftwareModule(), item),
      );
      const savedItem = await this.repo.findOne(newItem.id);
      return this.findOne(savedItem.id);
    }

    throw new HttpException('Software not found', HttpStatus.NOT_FOUND);
  }

  async updateOne(id: string, changes: any): Promise<SoftwareModuleDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      const udpatedItem = Object.assign(foundItem, changes);
      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }
}
