/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Subscriptions Controller
 *
 * Handles routes for Subscriptions.
 */

import {
  Body,
  Controller,
  Delete,
  Get,
  Param,
  Patch,
  Post,
  Query,
  Req,
  UseGuards,
} from '@nestjs/common';
import { SubscriptionsService } from './subscriptions.service';
import { SubscriptionCreateDto } from './subscriptions.dto';
import { ApiTags } from '@nestjs/swagger';
import { AuthGuard, AuthProfiles, AuthPublic, AuthRoles } from '@auth';
import { UserProfileType, UserRoleType } from '@shared/domain';

@ApiTags('Subscriptions')
@Controller('subscriptions')
export class SubscriptionsController {
  constructor(public service: SubscriptionsService) {}

  @Get('withTotals')
  getManyWithTotals(
    @Query('search') search: string,
    @Query('order') order: string,
  ) {
    return this.service.find(search, order);
  }

  // Returns the list of Subscriptions from a remote system.
  @Get('system')
  getFromSystem(@Req() req) {
    return this.service.findForSystem(req.user.id);
  }

  @Get(':id')
  getOne(@Param('id') id: string) {
    return this.service.findOne(id);
  }

  @Get('system/:systemId')
  getForSystem(@Param('systemId') systemId: string) {
    return this.service.findForSystem(systemId);
  }

  @Post()
  @AuthProfiles(UserProfileType.INNERSPEC)
  @AuthRoles(UserRoleType.MANAGER, UserRoleType.SUPERUSER)
  @UseGuards(AuthGuard)
  create(@Body() item: SubscriptionCreateDto) {
    return this.service.addOne(item);
  }

  @Post('purchase')
  @AuthProfiles(UserProfileType.USER)
  @AuthRoles(UserRoleType.MANAGER, UserRoleType.CORPORATE_MANAGER)
  @UseGuards(AuthGuard)
  purchase(@Body() item: SubscriptionCreateDto) {
    return this.service.purchaseOne(item);
  }

  @AuthPublic()
  @Post('process-purchase')
  processPayment(@Body('token') token: string) {
    // The user has accepted to pay.
    // Here is where we really execute the payment.
    return this.service.processPurchase(token);
  }

  @Patch(':id')
  update(@Param('id') id: string, @Body() changes: SubscriptionCreateDto) {
    return this.service.updateOne(id, changes);
  }

  @Delete(':id')
  deleteOne(@Param() id: string) {
    return this.service.deleteOne(id);
  }
}
