import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { Subscription, System, SoftwareModule, Payment } from '@shared/domain';
import {
  SubscriptionDto,
  SubscriptionCreateDto,
  SubscriptionListDto,
  SubscriptionListItemDto,
} from './subscriptions.dto';
import { HtmlTemplateService } from '@shared/html-template';
import { MailService } from '@shared/mail/mail.service';
import { PaymentWriteDto, PaymentService } from '@shared/payment';
import { ConfigService } from '@nestjs/config';
import { REQUEST } from '@nestjs/core';

@Injectable()
export class SubscriptionsService {
  private _clientUrl: string;

  baseSelectQuery = `
   SELECT 
        subscription.*,
        \`system\`.id AS systemId, \`system\`.sn AS systemSN,
        software_module.id AS software_moduleId, software_module.name AS softwareModuleName, 
        software_module.pn as softwareModulePN 
      FROM subscription
      LEFT JOIN \`system\` ON subscription.systemId = \`system\`.id
      LEFT JOIN software_module ON subscription.softwareModuleId = software_module.id
`;

  constructor(
    private _config: ConfigService,
    @Inject(REQUEST) private _request,
    @InjectRepository(Subscription) public repo: Repository<Subscription>,
    @InjectRepository(System) public systemRepo: Repository<System>,
    @InjectRepository(SoftwareModule)
    public softwareModuleRepo: Repository<SoftwareModule>,
    @InjectRepository(Payment) public paymentRepo: Repository<Payment>,
    private _htmlTemplate: HtmlTemplateService,
    private _mail: MailService,
    private _paymentService: PaymentService,
  ) {
    this._clientUrl = this._config.get('client.url');
  }

  async find(search: string, order: string): Promise<SubscriptionListDto> {
    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE subscription.id LIKE ?
      GROUP BY subscription.id
      ORDER BY subscription.id ${order}
      `,
      [s, order],
    );
    return plainToClass(SubscriptionListDto, {
      total: items.length,
      data: items,
    });
  }

  async findOne(id: string): Promise<SubscriptionDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE subscription.id LIKE ?
      GROUP BY subscription.id
      `,
      [id],
    );
    return plainToClass(SubscriptionDto, items[0]);
  }

  async findForSystem(systemId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE subscription.systemId = ? 
        ORDER BY subscription.id ASC
      `,
      [systemId],
    );
    return plainToClass(SubscriptionListItemDto, items);
  }

  async addOne(item: SubscriptionCreateDto): Promise<SubscriptionDto> {
    const canAddSubscription = await this.checkCanAddSubscription(item);
    const [foundSystem, foundSoftwareModule] = canAddSubscription;

    // Everything OK, create subscription.
    const newSubscription = await this.repo.save(
      Object.assign(new Subscription(), item),
    );

    // Send email to:
    // The user who added the subscription.
    // All the users who have access to the system.

    const systemAccounts = await foundSystem.systemAccounts;

    systemAccounts.forEach(async (account) => {
      const user = await account.user;

      const msg = this._htmlTemplate.toString(
        'subscriptions/mail/registration.html',
        {
          name: user.name || 'Hello',
          sn: foundSystem.sn,
          systemId: foundSystem.id,
          module: foundSoftwareModule.name,
          recurrence: newSubscription.type,
        },
      );
      this._mail.sendEmail(user.email, 'Activate Subscription', msg);
    });

    // Return record
    return this.findOne(newSubscription.id);
  }

  async purchaseOne(item: SubscriptionCreateDto): Promise<any> {
    const canAddSubscription = await this.checkCanAddSubscription(item);
    const [foundSystem, foundSoftwareModule] = canAddSubscription;
    const price = foundSoftwareModule[item.type];
    const payment = new PaymentWriteDto();

    payment.currency = 'USD';
    payment.description = `${item.type} subscription`.toUpperCase();
    payment.total = price.toString();
    payment.softwareModuleName = `${foundSoftwareModule.name} - P/N ${foundSoftwareModule.pn}`;
    payment.softwareModuleId = foundSoftwareModule.id;
    payment.systemId = foundSystem.id;

    const returnUrl = `${this._clientUrl}/purchase-confirmation`;
    const confirmationURL = await this._paymentService.createPayment(
      payment,
      returnUrl,
    );

    return { url: confirmationURL.href };
  }

  async processPurchase(token: string): Promise<any> {
    const paymentId = await this._paymentService.executePayment(token);

    const payment = await this.paymentRepo.findOne({
      where: { id: paymentId },
      relations: ['system', 'softwareModule'],
    });
    if (!payment) {
      throw new HttpException(
        'Payment information not found after it was processed',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }

    const system = payment.system;
    const softwareModule = payment.softwareModule;

    const newSubscription = new SubscriptionCreateDto();
    newSubscription.system = system.id;
    newSubscription.softwareModule = softwareModule.id;
    newSubscription.type = 'weekly';

    console.log(this._request);

    return await this.addOne(newSubscription);
  }

  async checkCanAddSubscription(
    item: SubscriptionCreateDto,
  ): Promise<[System, SoftwareModule]> {
    // Check that the system and software module passed as parameters of the
    // new subscription exist.
    const foundSystem = await this.systemRepo.findOne(item.system);
    if (foundSystem == null) {
      throw new HttpException('System not found.', HttpStatus.NOT_FOUND);
    }

    const foundSoftwareModule = await this.softwareModuleRepo.findOne(
      item.softwareModule,
    );
    if (foundSoftwareModule == null) {
      throw new HttpException(
        'Software Module not found.',
        HttpStatus.NOT_FOUND,
      );
    }

    // Check that the system doesn't have already an active subscription for
    // the requested module.
    const foundSubscription = await this.repo.findOne({
      where: {
        system: foundSystem,
        softwareModule: foundSoftwareModule,
        isActive: true,
      },
    });
    if (foundSubscription) {
      throw new HttpException(
        'The system already has an active subscription for the requested Software Module.',
        HttpStatus.BAD_REQUEST,
      );
    }

    return [foundSystem, foundSoftwareModule];
  }

  async updateOne(id: string, changes: any): Promise<SubscriptionDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      const udpatedItem = Object.assign(foundItem, changes);
      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  async deleteOne(id: string): Promise<any> {
    const subscription = await this.repo.findOne(id);
    const system = await subscription.system;
    const softwareModule = await subscription.softwareModule;
    const systemAccounts = await system.systemAccounts;

    // Send an email to everyone informing that the subscription
    // has ended.

    systemAccounts.forEach(async (account) => {
      const user = await account.user;

      const msg = this._htmlTemplate.toString(
        'subscriptions/mail/removal.html',
        {
          name: user.name || 'Hello',
          sn: system.sn,
          systemId: system.id,
          module: softwareModule.name,
          recurrence: subscription.type,
        },
      );
      this._mail.sendEmail(user.email, 'Subscription Ended', msg);
    });
    return this.repo.delete(id);
  }
}
