import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { SystemAccount, User, System } from '@shared/domain';
import { SystemAccountDto, SystemAccountCreateDto } from './systemAccounts.dto';
import { HtmlTemplateService } from '@shared/html-template';
import { MailService } from '@shared/mail/mail.service';

@Injectable()
export class SystemAccountsService {
  baseSelectQuery = `
    SELECT 
      system_account.*,
      user.id AS userId, user.email AS userEmail, user.password AS userPassword, user.name AS userName,
      user.avatar AS userAvatar, 
      \`system\`.id AS systemId, \`system\`.tag AS systemTag, \`system\`.sn AS systemSN,
      product.id AS productId, product.name AS productName
      FROM system_account
      LEFT JOIN user ON system_account.userId = user.id
      LEFT JOIN \`system\` ON system_account.systemId = \`system\`.id
      LEFT JOIN product ON \`system\`.productId = product.id
`;

  constructor(
    @InjectRepository(SystemAccount) public repo: Repository<SystemAccount>,
    @InjectRepository(User) public userRepo: Repository<User>,
    @InjectRepository(System) public systemRepo: Repository<System>,
    private _htmlTemplate: HtmlTemplateService,
    private _mail: MailService,
  ) {}

  async find(systemId: string): Promise<SystemAccountDto[]> {
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE system_account.systemId = ?
      ORDER BY user.email ASC
      `,
      [systemId],
    );
    return plainToClass(SystemAccountDto, items);
  }

  async findOne(id: string): Promise<SystemAccountDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE system_account.id = ?
      `,
      [id],
    );
    return plainToClass(SystemAccountDto, items[0]);
  }

  async addOne(item: SystemAccountCreateDto): Promise<SystemAccountDto> {
    // Check that the user and system passed as values exist.
    const foundUser = await this.userRepo.findOne(item.user);
    if (foundUser == null) {
      throw new HttpException('User not found', HttpStatus.NOT_FOUND);
    }

    const foundSystem = await this.systemRepo.findOne(item.system);
    if (foundSystem == null) {
      throw new HttpException('System not found', HttpStatus.NOT_FOUND);
    }

    const newSystemAccount = await this.repo.save(
      Object.assign(new SystemAccount(), item),
    );

    // Send email
    const msg = this._htmlTemplate.toString(
      'systemAccounts/mail/registration.html',
      {
        name: foundUser.name || 'Dear, ',
        systemId: foundSystem.id,
        sn: foundSystem.sn,
        level: item.level.toUpperCase(),
      },
    );
    this._mail.sendEmail(foundUser.email, 'System Account Activated', msg);

    // Return record
    return this.findOne(newSystemAccount.id);
  }

  async updateOne(id: string, changes: any): Promise<SystemAccountDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      const udpatedItem = Object.assign(foundItem, changes);
      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  async deleteOne(id: string): Promise<any> {
    const item = await this.repo.findOne(id);
    const user = await item.user;
    const system = await item.system;

    await this.repo.delete(id);

    // Send email
    const msg = this._htmlTemplate.toString(
      'systemAccounts/mail/removal.html',
      {
        name: user.name || 'Hello',
        systemId: system.id,
        sn: system.sn,
        level: item.level.toUpperCase(),
      },
    );
    this._mail.sendEmail(user.email, 'System Account Disabled', msg);

    return;
  }

  async findForUser(userId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE user.id = ?
        ORDER BY user.email ASC
      `,
      [userId],
    );
    return plainToClass(SystemAccountDto, items);
  }
}
