import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { SystemStatus, System } from '@shared/domain';
import { SystemStatusDto, SystemStatusCreateDto } from './systemStatus.dto';
import { REQUEST } from '@nestjs/core';
import { count } from 'rxjs/operators';

@Injectable()
export class SystemStatusService {
  baseSelectQuery = `
    SELECT 
      system_status.*,
      \`system\`.id AS systemID, \`system\`.sn AS systemSN,
      product.id AS productId, product.name AS productName
    FROM system_status
    LEFT JOIN \`system\` ON \`system\`.id = system_status.systemId
    LEFT JOIN product ON \`system\`.productId = product.id
`;

  constructor(
    @Inject(REQUEST) private _request,
    @InjectRepository(SystemStatus) public repo: Repository<SystemStatus>,
    @InjectRepository(System) public systemRepo: Repository<System>,
  ) {}

  async findForAll(date: string): Promise<SystemStatusDto[]> {
    // TODO: Replace with a direct SQL query.
    // THIS CODE IS NOT EFFICIENT, the more systems are added
    // in the site, the more processing this routine will take.
    // This code is for prototyping only!

    // Find unique systems in the status table.
    const systems: any[] = await this.systemRepo.query(`
      SELECT DISTINCT system_status.systemId
      FROM system_status    
    `);

    // For each systemId, get their latest status entry.
    const itemQueries = systems.map(
      async (system) =>
        await this.repo.query(
          this.baseSelectQuery +
            `
        WHERE system_status.systemId = ? AND DATE(system_status.time) = DATE(?)
        GROUP BY system_status.id
        ORDER BY system_status.time DESC
        LIMIT 1
        `,
          [system.systemId, date],
        ),
    );

    const items: any[] = await Promise.all(itemQueries);

    // From Promise.All we get an array containing the arrays resultant
    // from the SQL queries. We need to flatten them to one
    // single array before mapping to DTO and sending to client.
    const flattenItems: any[] = [].concat.apply([], items);

    return plainToClass(SystemStatusDto, flattenItems);
  }

  async findForOne(
    systemId: string,
    date: string,
    lastOnly: boolean,
  ): Promise<SystemStatusDto[]> {
    let query =
      this.baseSelectQuery +
      `
      WHERE \`system\`.id = ? AND DATE(system_status.time) = DATE(?)
      GROUP BY system_status.id
      ORDER BY system_status.id DESC
      `;

    if (lastOnly) {
      query += ' LIMIT 1';
    }

    const items: any[] = await this.repo.query(query, [systemId, date]);

    return plainToClass(SystemStatusDto, items);
  }

  async addOne(items: SystemStatusCreateDto[]): Promise<any> {
    // Extract system id from the current request
    const loggedSystem = this._request.user;
    if (!loggedSystem) {
      throw new HttpException(
        'No system session has been found in the current request',
        HttpStatus.NOT_FOUND,
      );
    }

    // Check the system exists
    const system = await this.systemRepo.findOne(loggedSystem.id);
    if (!system) {
      throw new HttpException(
        `System with id ${loggedSystem.id} not found`,
        HttpStatus.NOT_FOUND,
      );
    }

    // We have everything we need. We are ready to store the status.
    const updateItems = items.map(async (status) => {
      const newStatus = Object.assign(new SystemStatus(), status);
      newStatus.system = system;
      await this.repo.save(newStatus);
    });

    await Promise.all(updateItems);

    return { added: items.length };
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }
}
