import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import { System, Product, Location } from '@shared/domain';
import {
  SystemDto,
  SystemCreateDto,
  SystemListDto,
  SystemListItemDto,
} from './systems.dto';
import { HtmlTemplateService } from '@shared/html-template';
import { MailService } from '@shared/mail/mail.service';
import { AuthUserDataDto } from '@auth';
import { UserProfileType } from '@shared/domain';
import { REQUEST } from '@nestjs/core';

@Injectable()
export class SystemsService {
  baseSelectQuery = `
   SELECT 
        \`system\`.*,
        product.id AS productId, product.name AS productName,
        software.id AS softwareId, software.name AS softwareName,
        location.id AS locationId, location.name AS locationName,
        company.id AS companyId, company.name AS companyName
      FROM \`system\`
      LEFT JOIN product ON \`system\`.productId = product.id
      LEFT JOIN software ON product.softwareId = software.id
      LEFT JOIN location ON \`system\`.locationId = location.id
      LEFT JOIN company ON location.companyId = company.id
`;

  constructor(
    @Inject(REQUEST) private _request,
    @InjectRepository(System) public repo: Repository<System>,
    @InjectRepository(Product) public productRepo: Repository<Product>,
    @InjectRepository(Location) public locationRepo: Repository<Location>,
    private _htmlTemplate: HtmlTemplateService,
    private _mail: MailService,
  ) {}

  async find(search: string, order: string): Promise<SystemListDto> {
    // If the user is not staff, filter our all locations that are
    // not part of the parent company.

    const user = this._request.user;
    const forCompanyWhere =
      user.profile !== UserProfileType.INNERSPEC
        ? `AND company.id = '${user.companyId}'`
        : '';

    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE \`system\`.sn LIKE ? ${forCompanyWhere}
      GROUP BY \`system\`.id, \`system\`.sn
      ORDER BY \`system\`.sn ${order}
      `,
      [s, order],
    );
    return plainToClass(SystemListDto, { total: items.length, data: items });
  }

  async findOne(id: string): Promise<SystemDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
      WHERE \`system\`.id LIKE ?
      GROUP BY \`system\`.id, \`system\`.sn
      `,
      [id],
    );
    return plainToClass(SystemDto, items[0]);
  }

  async count(): Promise<number> {
    return this.repo.count();
  }

  async addOne(
    item: SystemCreateDto,
    author: AuthUserDataDto,
  ): Promise<SystemDto> {
    // Check that the product and location passed as values exist.
    const foundProduct = await this.productRepo.findOne(item.product);
    if (foundProduct == null) {
      throw new HttpException('Product not found', HttpStatus.NOT_FOUND);
    }

    const foundLocation = await this.locationRepo.findOne(item.location);
    if (foundLocation == null) {
      throw new HttpException('Location not found', HttpStatus.NOT_FOUND);
    }

    // Generate password.
    const password = this.randomPassword();
    const newItem = new System();
    newItem.password = password;

    // Save new system
    const newSystem = await this.repo.save(Object.assign(newItem, item));

    // Send email
    const email = this._htmlTemplate.toString(
      'systems/mail/registration.html',
      {
        name: author.name || 'Dear',
        sn: newSystem.sn,
        tag: newSystem.tag,
        password,
        systemId: newSystem.id,
      },
    );
    this._mail.sendEmail(author.email, 'Activate New System', email);

    // Return record
    return this.findOne(newSystem.id);
  }

  async updateOne(id: string, changes: any): Promise<SystemDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      const udpatedItem = Object.assign(foundItem, changes);
      const p = await this.repo.save(udpatedItem);
      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  deleteOne(id: string): Promise<any> {
    return this.repo.delete(id);
  }

  async findforDashboard(): Promise<any> {
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      GROUP BY \`system\`.id, \`system\`.sn
      ORDER BY \`system\`.id DESC
      LIMIT 5
      `,
    );
    return plainToClass(SystemListItemDto, items);
  }

  async findForCompany(companyId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE location.companyId = ?
        ORDER BY company.name ASC
      `,
      [companyId],
    );
    return plainToClass(SystemListItemDto, items);
  }

  async findForLocation(locationId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE \`system\`.locationId = ?
        ORDER BY location.name ASC
      `,
      [locationId],
    );
    return plainToClass(SystemListItemDto, items);
  }

  async resetSecret(id: string): Promise<SystemDto> {
    // Check that the product and location passed as values exist.
    const foundSystem = await this.repo.findOne(id);
    if (foundSystem == null) {
      throw new HttpException('System not found', HttpStatus.NOT_FOUND);
    }

    // Generate password.
    const password = this.randomPassword();
    foundSystem.password = password;

    await this.repo.save(foundSystem);

    // Send email
    const currentUser = this._request.user;

    const email = this._htmlTemplate.toString(
      'systems/mail/reset-secret.html',
      {
        name: currentUser.name || 'Hello,',
        sn: foundSystem.sn,
        tag: foundSystem.tag,
        password,
      },
    );
    this._mail.sendEmail(currentUser.email, 'Reset Asset Secret', email);

    // Return record
    return this.findOne(id);
  }

  randomPassword(): string {
    const passAt =
      'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    const passArray = Array.from({ length: 8 });

    return passArray
      .map(() => passAt.charAt(Math.random() * passAt.length))
      .join('');
  }
}
