/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Users Controller
 */

import {
  Body,
  Controller,
  Delete,
  Get,
  Param,
  Patch,
  Post,
  Query,
  UploadedFile,
  UseInterceptors,
} from '@nestjs/common';
import { UsersService } from './users.service';
import {
  UserCreateDto,
  UserResetPasswordDto,
  UserUpdateDto,
} from './users.dto';
import { ApiTags } from '@nestjs/swagger';
import { FileInterceptor } from '@nestjs/platform-express';
import { AuthPublic } from '@auth';

@ApiTags('Users')
@Controller('users')
export class UsersController {
  constructor(public service: UsersService) {}

  @Get('count')
  getCount() {
    return this.service.count();
  }

  @Get('withTotals')
  getManyWithTotals(
    @Query('search') search: string,
    @Query('order') order: string,
  ) {
    return this.service.find(search, order);
  }

  @Get(':id')
  getOne(@Param('id') id: string) {
    return this.service.findOne(id);
  }

  @Get('dashboard/:id')
  getForDashboard() {
    return this.service.findForDashboard();
  }

  @Get('online/:id')
  getForDashboardOnline() {
    return this.service.findForOnline();
  }

  @Get('location/:locationId')
  getForLocation(@Param('locationId') locationId: string) {
    return this.service.findForLocation(locationId);
  }

  @Get('company/:companyId')
  getForCompany(@Param('companyId') companyId: string) {
    return this.service.findForCompany(companyId);
  }

  @Get('system/:systemId')
  getForSystem(@Param('systemId') systemId: string) {
    return this.service.findForSystem(systemId);
  }

  @Post()
  create(@Body() item: UserCreateDto) {
    return this.service.addOne(item);
  }

  @Patch(':id')
  @UseInterceptors(FileInterceptor('file'))
  update(
    @Param('id') id: string,
    @Body() changes: UserUpdateDto,
    @UploadedFile() file: Express.Multer.File,
  ) {
    // If there is no image sent, omit it from the update.
    const data = { ...changes };
    if (file) {
      data.avatar = file.filename;
    }

    return this.service.updateOne(id, data);
  }

  @Post('resetPassword')
  @AuthPublic()
  resetPassword(@Body() values: UserResetPasswordDto) {
    return this.service.resetPassword(values);
  }

  @Delete(':id')
  deleteOne(@Param() id: string) {
    return this.service.deleteOne(id);
  }
}
