/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * DTOs for Users.
 */

import { Exclude, Expose, Type } from 'class-transformer';
import {
  IsNotEmpty,
  IsOptional,
  IsEnum,
  IsEmail,
  Length,
  Matches,
  MaxLength,
  ValidateIf,
} from 'class-validator';
import { IsSameAs } from '@shared/validators';
import { UserRoleType } from '@shared/domain';

/**
 * DTO for List View in the front-end.
 */
@Exclude()
export class UsersListDto {
  @Expose()
  total: number;

  @Expose()
  @Type(() => UsersListItemDto)
  data: UsersListItemDto[];
}

/**
 * DTO for each item for the List View in the front-end.
 */
@Exclude()
export class UsersListItemDto {
  @Expose()
  id: string;

  @Expose()
  email: string;

  @Expose()
  name: string;

  @Expose()
  avatar: string;

  @Expose()
  locationId: string;

  @Expose()
  locationName: string;

  @Expose()
  companyId: string;

  @Expose()
  companyName: string;

  @Expose()
  profile: string;

  @Expose()
  role: string;

  @Expose()
  totalSystemAccounts: number;

  @Expose()
  isActive: boolean;
}

/**
 * DTO for Detail View in the front-end.
 */
@Exclude()
export class UserDto {
  @Expose()
  id: string;

  @Expose()
  authorId: string;
  @Expose()
  authorName: string;

  @Expose()
  email: string;

  @Expose()
  name: string;

  @Expose()
  profile: string;

  @Expose()
  role: string;

  @Expose()
  title: string;

  @Expose()
  department: string;

  @Expose()
  phone: string;

  @Expose()
  avatar: string;

  @Expose()
  locationId: string;
  @Expose()
  locationName: string;

  @Expose()
  companyId: string;
  @Expose()
  companyName: string;

  @Expose()
  totalSystemAccount: number;

  @Expose()
  created: string;

  @Expose()
  updated: string;

  @Expose()
  description: string;

  @Expose()
  notes: string;
}

/**
 * DTO for creation and update using data sent from the front-end.
 * Includes decorators for validation.
 */
export class UserCreateDto {
  @IsEmail()
  email: string;

  @IsEnum(UserRoleType)
  role: UserRoleType;

  @IsNotEmpty()
  location: string;
}

export class UserUpdateDto {
  @IsOptional()
  @IsEmail()
  email?: string;

  // Password must be at least 8 character long,
  // and must contain one upper case letter,
  // one lower case letter, one digit, and one
  // special character
  @IsOptional()
  @Length(8, 30)
  @Matches(/^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[#?!@$%^&*-]).{8,}$/, {
    message: 'Password is too weak',
  })
  password?: string;

  @IsOptional()
  @ValidateIf((o) => o.password)
  @Length(8, 30)
  @IsSameAs('password', { message: "Passwords don't match" })
  confirmPassword?: string;

  @IsOptional()
  @Length(3, 60)
  name: string;

  @IsOptional()
  @IsEnum(UserRoleType)
  role?: UserRoleType;

  @IsOptional()
  @MaxLength(60)
  title?: string;

  @IsOptional()
  @MaxLength(60)
  department?: string;

  @IsOptional()
  @Matches(
    /(\+\d{1,3}\s?)?((\(\d{3}\)\s?)|(\d{3})(\s|-?))(\d{3}(\s|-?))(\d{4})(\s?(([E|e]xt[:|.|]?)|x|X)(\s?\d+))?/,
    {
      message: 'Phone number has wrong format',
    },
  )
  phone?: string;

  @IsOptional()
  avatar: string;

  @IsOptional()
  location?: string;
}

export class UserResetPasswordDto {
  @IsEmail()
  email: string;
}
