/**
 * NDT-Link Backend.
 * Copyright 2021, Chakir Mrabet - Innerspec Technologies, Inc.
 *
 * Users Service
 */

import { HttpException, HttpStatus, Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { plainToClass } from 'class-transformer';
import { Repository } from 'typeorm';
import * as generatePassword from 'generate-password';
import { HtmlTemplateService } from '@shared/html-template';
import { MailService } from '@shared/mail/mail.service';
import { User, Location, Session } from '@shared/domain';
import {
  UserDto,
  UserCreateDto,
  UsersListDto,
  UsersListItemDto,
  UserResetPasswordDto,
} from './users.dto';
import { objectClean } from '@shared/infrastructure';
import { UserProfileType } from '@shared/domain';
import { REQUEST } from '@nestjs/core';
import { ConfigService } from '@nestjs/config';
import * as fs from 'fs';
import { join } from 'path';

@Injectable()
export class UsersService {
  baseSelectQuery = `
    SELECT
      user.*, 
      location.id AS locationId, company.id AS companyId,
      location.name AS locationName, company.name AS companyName,
      COUNT(system_account.id) AS totalSystemAccounts,
      if(author.name is null, author.email, author.name) AS authorName
    FROM user
    LEFT JOIN location ON user.locationId = location.id
    LEFT JOIN company ON location.companyId = company.id
    LEFT JOIN system_account ON system_account.userId = user.id
    LEFT JOIN user AS author ON author.id = user.authorId
  `;

  constructor(
    @Inject(REQUEST) private _request,
    @InjectRepository(User) public repo: Repository<User>,
    @InjectRepository(Location) public locationsRepo: Repository<Location>,
    @InjectRepository(Session) public sessionsRepo: Repository<Session>,
    private _config: ConfigService,
    private _htmlTemplate: HtmlTemplateService,
    private _mail: MailService,
  ) {}

  async find(search: string, order: string): Promise<UsersListDto> {
    // If the user is not staff, filter our all locations that are
    // not part of the parent company.

    const user = this._request.user;
    const forCompanyWhere =
      user.profile !== UserProfileType.INNERSPEC
        ? `AND company.id = '${user.companyId}'`
        : '';

    const s = `%${search || ''}%`;
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE user.name LIKE ? OR user.email LIKE ? ${forCompanyWhere}
        GROUP BY user.id
        ORDER BY user.email ${order}, user.name ${order}
      `,
      [s, s],
    );
    return plainToClass(UsersListDto, { total: items.length, data: items });
  }

  async findOne(id: string): Promise<UserDto> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE user.id = ?
      `,
      [id],
    );

    return plainToClass(UserDto, items[0]);
  }

  async count(): Promise<number> {
    return this.repo.count();
  }

  async addOne(item: UserCreateDto): Promise<UserDto> {
    const foundLocation = await this.locationsRepo.findOne(item.location);

    if (foundLocation) {
      // If the location is one from Innerpec company, the new account
      // should be for a staff member rather than a user member.
      const company = await foundLocation.company;
      const profile =
        company.id === this._config.get('data.innerspecCompanyId')
          ? UserProfileType.INNERSPEC
          : UserProfileType.USER;

      // Generate a new random password
      const password = this.generatePassword();

      const newUser = Object.assign(new User(), { ...item, password, profile });
      const savedUser = await this.repo.save(newUser);

      // Send email
      const msg = this._htmlTemplate.toString('users/mail/registration.html', {
        email: savedUser.email,
        password,
      });
      this._mail.sendEmail(savedUser.email, 'Welcome to NDT-Link', msg);

      // Return data for the new saved user.
      const savedItem = await this.repo.findOne(savedUser.id);
      return this.findOne(savedItem.id);
    }

    throw new HttpException('Location not found', HttpStatus.NOT_FOUND);
  }

  async updateOne(id: string, changes: any): Promise<UserDto> {
    const foundItem = await this.repo.findOne(id);
    if (foundItem) {
      // Reset the password so it doesn't get encrypted again by the
      // entity's @BeforeUpdate listener.
      delete foundItem.password;

      // If the avatar has changed, delete the previous one.
      if (foundItem.avatar) {
        this.deleteAvatar(foundItem);
      }

      // Create a new user with changed properties sent from the client.
      // If the changed properties include a password, then it will be
      // encrypted automatically by the @BeforeUpdate listener.
      const updatedItem = Object.assign(foundItem, objectClean(changes));
      const p = await this.repo.save(updatedItem);

      // If the changes contained a password then it has been changed. Send
      // an notification email to the user.
      if (changes.hasOwnProperty('password')) {
        const msg = this._htmlTemplate.toString(
          'users/mail/passwordChanged.html',
          {
            name: foundItem.name || 'Hello,',
            helpEmail: this._config.get('data.helpEmail'),
          },
        );
        this._mail.sendEmail(foundItem.email, 'Password Changed', msg);
      }

      return this.findOne(p.id);
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  async resetPassword(values: UserResetPasswordDto): Promise<any> {
    // Find a user with the given email.
    const { email } = values;
    const foundUser = await this.repo.findOne({ where: { email } });
    if (!foundUser) {
      throw new HttpException(
        `There is no user with email "${email}"`,
        HttpStatus.NOT_FOUND,
      );
    }

    // Generate a new random password
    const password = this.generatePassword();

    // Update the user with the new password
    foundUser.password = password;
    await this.repo.save(foundUser);

    // Send email
    const msg = this._htmlTemplate.toString('users/mail/resetPassword.html', {
      name: foundUser.name || 'Hello',
      email: email,
      password,
    });
    this._mail.sendEmail(email, 'Reset Password', msg);
  }

  async deleteOne(id: string): Promise<any> {
    const foundItem = await this.repo.findOne(id);

    if (foundItem) {
      // Do not delete if it's a system based account
      if (foundItem.isSystem) {
        throw new HttpException(
          `${foundItem.email} is a system based account and cannot be deleted`,
          HttpStatus.BAD_REQUEST,
        );
      }

      const deleted = await this.repo.delete(id);

      // TODO: refactor this into an external service to manage files.
      if (deleted && foundItem.avatar) {
        this.deleteAvatar(foundItem);
      }

      return deleted;
    }

    throw new HttpException('Not Found', HttpStatus.NOT_FOUND);
  }

  async findForDashboard(): Promise<any> {
    const items: any[] = await this.repo.query(
      this.baseSelectQuery +
        `
      GROUP BY user.id
      ORDER BY user.email ASC, user.name ASC
      LIMIT 5
      `,
    );
    return plainToClass(UsersListItemDto, items);
  }

  /**
   * Returns all the users currently connected to NDT-Link
   */
  async findForOnline(): Promise<UsersListItemDto[]> {
    const items: any[] = await this.sessionsRepo.query(
      `
        SELECT user.*
        FROM user
        LEFT JOIN session ON session.userId = user.id
        WHERE session.userId = user.id 
        GROUP BY user.id
        ORDER BY user.email ASC, user.name ASC
      `,
    );
    return plainToClass(UsersListItemDto, items);
  }

  async findForLocation(locationId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE user.locationId = ?
        GROUP BY user.id
        ORDER BY user.email ASC, user.name ASC
      `,
      [locationId],
    );
    return plainToClass(UsersListItemDto, items);
  }

  async findForCompany(companyId: string): Promise<any> {
    const items: any = await this.repo.query(
      this.baseSelectQuery +
        `
        WHERE company.id = ?
        GROUP BY user.id
        ORDER BY user.email ASC, user.name ASC
      `,
      [companyId],
    );
    return plainToClass(UsersListItemDto, items);
  }

  async findForSystem(systemId: string): Promise<any> {
    const items: any = await this.repo.query(
      `
      SELECT
        user.*, \`system\`.id AS systemId, \`system\`.locationId 
      FROM user
        LEFT JOIN \`system\` ON \`system\`.locationId = user.locationId
      WHERE \`system\`.id = ?
      GROUP BY user.id
      ORDER BY user.email ASC, user.name ASC
      `,
      [systemId],
    );
    return plainToClass(UsersListItemDto, items);
  }

  generatePassword(): string {
    return generatePassword.generate({
      length: 8,
      uppercase: true,
      symbols: true,
      numbers: true,
      strict: true,
    });
  }

  deleteAvatar(item: User) {
    const path = join(__dirname, '..', '..', 'upload');
    fs.unlinkSync(`${path}/${item.avatar}`);
  }
}
